unit DisassemblyListUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CompFormUnit, ExtCtrls, Grids, Menus, StringEditUnit;

type
  PPChar=^PChar;
  TDisasmProc = procedure (Address:Longint;var prograddress,opcode,disasmcode,sourcecode:PChar;var opcodelen:Longint) of object;
  TBreakPointProc = procedure (Address:Longint;var isbreakpoint:Boolean) of object;
  TGetIPProc = procedure (var ip:Longint) of object;
  PLongintArray=^TLongintArray;
  TLongintArray=Array[0..0] of Longint;
  TDisassemblyListInfo = class (TCompInfo)
    public
      IPList : PLongintArray;
      StringGrid1: TStringGrid;
      Offset,EndAddr : Longint;
      LastIP : Longint;
      AdSize : Longint;
      clIp,clBreak:TColor;
      clIpPen,clBreakPen:TColor;
      //Interface avec un programme de dcompilation
      DisasmProc : TDisasmProc;
      IsBreakPoint : TBreakPointProc;
      GetIP        : TGetIPProc;
      ToggleBreakPoint : TIntProc;
      ModifDialog:TForm;
      OnChangeIP: TStringProc;
      OnClearBreakPoints: TSimpleProc;
      OnRunToCursor : TIntProc;
      Constructor Create;
      Procedure SpreadDragDrop;override;
      Procedure SetMemRange(_Start,_End:Longint);
  //    Procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
      Procedure Update;
  end;

  TDisassemblyListViewer = class(TCompForm)
    StringGrid1: TStringGrid;
    PopupMenu1: TPopupMenu;
    Breakpoints1: TMenuItem;
    SetIP1: TMenuItem;
    Runtocursor1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure StringGrid1DrawCell(Sender: TObject; Col, Row: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Breakpoints1Click(Sender: TObject);
    procedure Runtocursor1Click(Sender: TObject);
    procedure SetIP1Click(Sender: TObject);
    procedure Clearbreakpoints1Click(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  DisassemblyListViewer: TDisassemblyListViewer;

implementation
var ImgList:TImageList;

{$R *.DFM}
{$R symbols.res}

Constructor TDisassemblyListInfo.Create;
Begin
  inherited;
  IPList:=nil;
End;

Procedure TDisassemblyListInfo.SpreadDragDrop;
Begin
  StringGrid1.OnStartDrag:=Control.OnStartDrag;
  StringGrid1.OnDragDrop:=Control.OnDragDrop;
  StringGrid1.OnEndDrag:=Control.OnEndDrag;
  StringGrid1.OnDragOver:=Control.OnDragOver;
End;

Procedure TDisassemblyListInfo.Update;
Var IP:Longint;
Begin
  GetIP(IP);
  if LastIP<>IP then
  Begin
    if (IPList<>nil) and (IP>=Offset) and (IP<EndAddr) then
    Begin
      if IPList[IP-Offset]<StringGrid1.TopRow then StringGrid1.TopRow:=IPList[IP-Offset];
      if IPList[IP-Offset]>StringGrid1.TopRow+StringGrid1.VisibleRowCount then
        StringGrid1.TopRow:=IPList[IP-Offset];
    End;
    //Refresh IP and lastIP lines.
    StringGrid1.Invalidate;
  End;
  LastIP:=IP;
End;

Procedure TDisassemblyListInfo.SetMemRange(_Start,_End:Longint);
Var progaddress,code,disasmcode,sourcecode:PChar;
    opcodelen,Address,i,j:Longint;
Begin
  Offset:=_Start;
  EndAddr:=_End;
  Address:=_Start;
  if (IPList<>nil) then FreeMem(IPList);
  GetMem(IPList,Sizeof(Longint)*(_End-_Start));
  i:=0;
  if Assigned(DisasmProc) then
  Begin
    Repeat
      DisasmProc(Address,progaddress,code,disasmcode,sourcecode,opcodelen);
      StringGrid1.Cells[0,i+1]:=progaddress;
      StringGrid1.Cells[1,i+1]:=code;
      StringGrid1.Cells[2,i+1]:=disasmcode;
      StringGrid1.Cells[3,i+1]:=sourcecode;
      for j:=0 to opcodelen-1 do
      Begin
        if Address+j<_End then IPList[Address+j-_Start]:=i+1;
      End;
      i:=i+1;
      StringGrid1.RowCount:=i+1;
      Address:=Address+opcodelen;
    Until Address>=_End;
  End;
End;


//******************************************************************************

procedure TDisassemblyListViewer.FormCreate(Sender: TObject);
Var P:TDisassemblyListInfo;
    t:integer;
begin
  inherited;
  StringGrid1.ColWidths[0]:=54;
  StringGrid1.ColWidths[1]:=35;
  P:=TDisassemblyListInfo.Create;
  Control.Tag:=longint(P);
  P.AdSize:=4;
  //couleurs par dfaut
  P.clIP:=clBlue;
  P.clBreak:=clRed;
  P.clIPPen:=clWhite;
  P.clBreakPen:=clBlack;
  P.DisasmProc:=nil;
  P.IsBreakPoint:=nil;
  P.StringGrid1:=StringGrid1;
  P.ModifDialog:=nil;
  for t:=0 to Componentcount-1 do
      Components[t].Tag:=longint(P);
end;

procedure TDisassemblyListViewer.StringGrid1DrawCell(Sender: TObject; Col,
  Row: Integer; Rect: TRect; State: TGridDrawState);
Var Ip,Address:Longint;
    bp:Boolean;
  Procedure HighLightLine(Color,PColor:TColor);
  Var x:integer;
  Begin
    with TDisassemblyListInfo(TComponent(Sender).Tag) do
    Begin
      //HighLight a line
      StringGrid1.Canvas.Brush.Color:=Color;
      StringGrid1.Canvas.Pen.Color:=Color;
      StringGrid1.Canvas.Font.Color:=PColor;
      if Col=0 then x:=Rect.Left+20 else x:=Rect.Left;
      StringGrid1.Canvas.Rectangle(x,Rect.Top,Rect.Right+1,Rect.Bottom);
    End;
  End;

  Procedure DrawIP;
  Begin
    with TDisassemblyListInfo(TComponent(Sender).Tag) do
      if Col=0 then ImgList.Draw(StringGrid1.Canvas,Rect.Left,Rect.Top+2,0);
  End;

  Procedure DrawBreak;
  Begin
    with TDisassemblyListInfo(TComponent(Sender).Tag) do
      if Col=0 then ImgList.Draw(StringGrid1.Canvas,Rect.Left+10,Rect.Top+2,1);
  End;

Begin
  With TDisassemblyListInfo(TComponent(Sender).Tag) do
  Begin
    if Row=0 then
    Begin
      StringGrid1.Canvas.Brush.Color:=StringGrid1.FixedColor;
      StringGrid1.Canvas.Pen.Color:=StringGrid1.FixedColor;
      StringGrid1.Canvas.Rectangle(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
      Case Col of
        0:StringGrid1.Canvas.TextOut(Rect.Left+3,Rect.Top,'Address');
        1:StringGrid1.Canvas.TextOut(Rect.Left+3,Rect.Top,'Code');
        2:StringGrid1.Canvas.TextOut(Rect.Left+3,Rect.Top,'Disassembly code');
        3:StringGrid1.Canvas.TextOut(Rect.Left+3,Rect.Top,'Source code');
      End;
    End else
    Begin
      StringGrid1.Canvas.Brush.Color:=StringGrid1.Color;
      StringGrid1.Canvas.Pen.Color:=StringGrid1.Color;
      StringGrid1.Canvas.Rectangle(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
      Address:=Offset+Row-1;
      bp:=false;
      if Assigned(IsBreakPoint) then
        IsBreakPoint(Address,bp);
      ip:=0;
      if Assigned(GetIp) then
        GetIp(ip);
      if Address=IP then
      Begin
        HighLightLine(clIp,ClIpPen);
        DrawIP;
      End else
      Begin
        if bp then
        HighLightLine(clBreak,clBreakPen);
      End;
      if bp then DrawBreak;
      Case col of
        0:StringGrid1.Canvas.TextOut(Rect.Left+23,Rect.Top,StringGrid1.Cells[Col,Row] );
        1,2,3:StringGrid1.Canvas.TextOut(Rect.Left+3,Rect.Top,StringGrid1.Cells[Col,Row] );
      End;
      StringGrid1.Canvas.Font.Color:=clBlack;
    End;
  End;
End;

procedure TDisassemblyListViewer.StringGrid1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  With TDisassemblyListInfo(TComponent(Sender).Tag) do
  Begin
    if Assigned(Control.OnMouseDown) then
      Control.OnMouseDown(Sender,Button,Shift,X,Y);
  End;
end;

procedure TDisassemblyListViewer.Breakpoints1Click(Sender: TObject);
Var i:integer;
begin
  inherited;
  With TDisassemblyListInfo(TComponent(Sender).Tag) do
  Begin
    if Assigned(ToggleBreakPoint) then
    Begin
      i:=Offset-1;
      repeat
        inc(i)
      until (i>=EndAddr-1) or (IPList[i-Offset]=StringGrid1.Row);
      if (IPList[i-Offset]=StringGrid1.Row) then ToggleBreakPoint(i);
    End;
    StringGrid1.Invalidate;
  End;
end;

procedure TDisassemblyListViewer.Runtocursor1Click(Sender: TObject);
Var i:integer;
begin
  inherited;
  With TDisassemblyListInfo(TComponent(Sender).Tag) do
  Begin
    if Assigned(OnRunToCursor) then
    Begin
      i:=Offset-1;
      repeat
        inc(i)
      until (i>=EndAddr-1) or (IPList[i-Offset]=StringGrid1.Row);
      if (IPList[i-Offset]=StringGrid1.Row) then OnRunToCursor(i);
    End;
  End;
end;

procedure TDisassemblyListViewer.SetIP1Click(Sender: TObject);
begin
  inherited;
  With TDisassemblyListInfo(TComponent(Sender).Tag) do
  Begin
    if ModifDialog=nil then
    Begin
      ModifDialog:=TStringEdit.Create(nil);
      TStringEdit(ModifDialog).OnValidate:=OnChangeIP;
    End;
    TStringEdit(ModifDialog).Caption:='Set IP';
    TStringEdit(ModifDialog).VarEdit.Text:=Format('%Xh',[LastIP]);
    ModifDialog.Show;
    ModifDialog.SetFocus;
  End;
end;

procedure TDisassemblyListViewer.Clearbreakpoints1Click(Sender: TObject);
begin
  inherited;
  With TDisassemblyListInfo(TComponent(Sender).Tag) do
  Begin
    if Assigned(OnClearBreakPoints) then  OnClearBreakPoints;
  End;  
end;

Initialization
  ImgList:=TImageList.Create(nil);
  ImgList.Height:=8;
  ImgList.Width:=8;
  ImgList.BkColor:=clNone;
  ImgList.ResourceLoad(rtbitmap,'DEBUGSYMBOLS',$C0C0C0);
end.
