/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#ifndef __PROJECT_H__
#include "project.h"
#endif
#include "TestUnit.h"
#include "about.h"
#include "WindowFormUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
int Refreshspeed;
int loop=10;
int loops=0;
int lastloop,loopmean;
HANDLE ParentHandle;

#define CM_SIMULATORMESSAGE (WM_USER + 400)
//---------------------------------------------------------------------------

void SendString(char* s)
{
  cout << s;
  char buffer[252];
  if (strlen(s)>249) s[249]=0;
  int t=strlen(s);
  strcpy((char*)(&buffer),s);
  buffer[t]='\r';
  buffer[t+1]='\n';
  buffer[t+2]=0;
  if (ParentHandle) {
    for (int i=0;i<(int)strlen(s);i+=6)
      SendMessage(ParentHandle,CM_SIMULATORMESSAGE,*((short*)&buffer[i]),*((int*)&buffer[i+2]));
  }
}

int Simulate(int usermsg)
{
  _control87(MCW_EM,MCW_EM);
  loops++;
  int res=Project.IdleLoop();
  if (usermsg) Application->ProcessMessages();
  /*if (Project.RunMode) Form1->StatusBar1->Panels->Items[1]->Text="Running"; else
    Form1->StatusBar1->Panels->Items[1]->Text="Stopped";
    */
  return res;
}

__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
  RegisterComponents();
  //Open the file
  if (ParamCount()>=1) {
    ParentHandle=0;
    if (ParamCount()>=2) {
      sscanf(ParamStr(2).c_str(),"%X",&ParentHandle);
    }
    OpenFile(ParamStr(1).c_str());
  }
  loops=0;
}

//------------------------------------------------------------------------------



void __fastcall TForm1::Run1Click(TObject *Sender)
{
  if (Project.RunMode==RUN_STOP) {
    Project.RunMode=RUN_ANIM+RUN_FASTANIM+RUN_TICK;
    while (Simulate(1)>=0);
  }
}
//------------------------------------------------------------------------------

void __fastcall TForm1::Break1Click(TObject *Sender)
{
  Project.RunMode=RUN_STOP;
}
//------------------------------------------------------------------------------
void __fastcall TForm1::OpenFile(char *name)
{
  FILE *parsein;
  Caption=name;
  parsein = fopen(name,"r");
  if (parsein == NULL)
    error(TYPE_ERROR,-1, NULL,"Can't open file '%s'",name);
  else {
    _control87(MCW_EM,MCW_EM);
    parse(parsein, name);
    Project.StartInterface();
  }
}

//------------------------------------------------------------------------------

void fatalerrorexit()
{
  exit(1);
}

void disperrormessage(int type, int line, char *File, char *msg, va_list val)
{
  char buffer[200];
  char fileinfo[200];
  if (line>=0) sprintf(buffer,"at line %d",line);
    else buffer[0]=0;
  if (File) sprintf(fileinfo,"in file %s %s",File,buffer);
    else fileinfo[0]=0;
  vsprintf(buffer,msg,val);
  switch (type) {
    case TYPE_ERROR:  {
      Form1->Memo1->Lines->Add(Format("Error: '%s' %s",ARRAYOFCONST((buffer,fileinfo))));
      SendString(Format("Error: '%s' %s",ARRAYOFCONST((buffer,fileinfo))).c_str());
      break;
    }
    case TYPE_FATALERROR: {
      MessageDlg(Format("Fatal Error: '%s' %s",ARRAYOFCONST((
        buffer,fileinfo))),mtError,TMsgDlgButtons() << mbOK,0);
      SendString(Format("Fatal Error: '%s' %s",ARRAYOFCONST((
        buffer,fileinfo))).c_str());
      break;
    }
    case TYPE_WARNING: {
      Form1->Memo1->Lines->Add(Format("Warning: '%s' %s",ARRAYOFCONST((
        buffer,fileinfo)))
      );
      SendString(Format("Warning: '%s' %s",ARRAYOFCONST((buffer,fileinfo))).c_str());
      break;
    }
  }
}

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
  loopmean=loops-lastloop;
  lastloop=loops;

  Form1->StatusBar1->Panels->Items[0]->Text=Format("%d loops, %d loops/s",ARRAYOFCONST((loops,loopmean)));
  /*if (Project.RunMode) Form1->StatusBar1->Panels->Items[1]->Text="Running"; else
    Form1->StatusBar1->Panels->Items[1]->Text="Stopped";
  */
}
//---------------------------------------------------------------------------



void __fastcall TForm1::Quit1Click(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::FormResize(TObject *Sender)
{
  NameEdit->Width=Width-NameEdit->Left-8;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::NameEditMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  NameEdit->BeginDrag(false);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::NameEditStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
  symbol* symb=Project.getsymbol(NameEdit->Text.c_str());
  if (symb) {
    DragObject=new WDragObject();
    ((WDragObject*)DragObject)->wi=symb->getwidget();
    ((WDragObject*)DragObject)->todestroy=1;
  } else {
    DragObject=new WDragObject();
    ((WDragObject*)DragObject)->wi=NULL;
    ((WDragObject*)DragObject)->todestroy=0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Timer2Timer(TObject *Sender)
{
  if (!active) {
    active=1;
    int i;
    while ((i=Simulate(1))==0) {};
    if (i<0) Timer2->Enabled=false;
    active=0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Newwindow1Click(TObject *Sender)
{
  TWindowForm* win=(TWindowForm*)(new window(Project.getattributes(), displaybox::getempty(BOX_V)->getwidget()))->control;
  win->AutosizeClick(Sender);
  win->Width=300;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Open1Click(TObject *Sender)
{
  if (OpenDialog->Execute()) {
    OpenFile(OpenDialog->FileName.c_str());
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::About1Click(TObject *Sender)
{
  AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
  //

}
//---------------------------------------------------------------------------

