/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "project.h"
#include "WindowFormUnit.h"
#include "clipbrd.hpp"
#include "animspeedunit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

//---------------------------------------------------------------------------
void __fastcall TWindowForm::GETMINMAXINFO(TMessage &Message)

{
 Message.Msg=0;
 LPMINMAXINFO lpmmi = (LPMINMAXINFO) Message.LParam;
 lpmmi->ptMinTrackSize.x=129;
 lpmmi->ptMinTrackSize.y=100;

 // TControl::CMChangeColor(Message);   // appelle le gestionnaire de message reu en hritage
}
//---------------------------------------------------------------------------
__fastcall TWindowForm::TWindowForm(TComponent* Owner)
    : TForm(Owner)
{
  Refreshspeed=1;
  closing=0;
}
//---------------------------------------------------------------------------
void __fastcall TWindowForm::OrganizeAutoClick(TObject *Sender)
{
 // OrganizeAuto->Checked=!OrganizeAuto->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TWindowForm::UpdateSpeed()
{
  switch (Refreshspeed) {
    case 1:
      Normal->Checked=true;
      break;
    case 2:
      Fast->Checked=true;
      break;
    case 3:
      Norefresh->Checked=true;
      break;
  }
}

void __fastcall TWindowForm::AutosizeClick(TObject *Sender)
{
  Autosize->Checked=!Autosize->Checked;
  if (Autosize->Checked) {
    BorderStyle=bsSingle;
    BorderIcons = (TBorderIcons()<< biSystemMenu << biMinimize);
  }
  else {
    BorderStyle=bsSizeable;
    BorderIcons = (TBorderIcons()<< biMaximize << biMinimize <<biSystemMenu);
  }
  OnResize(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::AlwaysontopClick(TObject *Sender)
{
  Alwaysontop->Checked=!Alwaysontop->Checked;
  if (Alwaysontop->Checked) FormStyle=fsStayOnTop;
    else FormStyle=fsNormal;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Newwindow1Click(TObject *Sender)
{
  new window(Project.getattributes(), displaybox::getempty(BOX_V)->getwidget());
}
//---------------------------------------------------------------------------


void __fastcall TWindowForm::Copytexttoclipboard1Click(TObject *Sender)
{
  TMemoryStream* stream=new TMemoryStream();
  win->exportwidget(stream,0);
  char b=0;
  stream->Write((void*)&b,1);
  Clipboard()->SetTextBuf(((char*)stream->Memory));
  delete stream;
}
//---------------------------------------------------------------------------



void __fastcall TWindowForm::NameEditMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  NameEdit->BeginDrag(false);
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::NameEditStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
  symbol* symb=Project.getsymbol(NameEdit->Text.c_str());
  if (symb) {
    DragObject=new WDragObject();
    ((WDragObject*)DragObject)->wi=symb->getwidget();
    ((WDragObject*)DragObject)->todestroy=1;
  } else {
    DragObject=new WDragObject();
    ((WDragObject*)DragObject)->wi=NULL;
    ((WDragObject*)DragObject)->todestroy=0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Panel1Resize(TObject *Sender)
{
  NameEdit->Width=Width-NameEdit->Left-8;;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::TrashDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
  // Destroy the widget dropped here
  symbolwidget *wi=(symbolwidget*)((WDragObject*)Source)->wi;
  widget *wiparent=wi->parent;
  //Remove the widget from its owner
  int destroyed=1;
  if (wiparent)
    destroyed=wiparent->RemoveChild(wi);
  if (!destroyed)
    wiparent->updatewindow();
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::HboxStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
  // Create a new HBox;
  DragObject=new WDragObject();
  ((WDragObject*)DragObject)->wi=displaybox::getempty(BOX_H)->getwidget();
  ((WDragObject*)DragObject)->todestroy=1;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::VboxStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
  //Create a New VBox
  DragObject=new WDragObject();
  ((WDragObject*)DragObject)->wi=displaybox::getempty(BOX_V)->getwidget();
  ((WDragObject*)DragObject)->todestroy=1;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::TrashDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
  Accept=true;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::NameEditDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
  Accept=true;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::NameEditDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
  symbolwidget *wi=(symbolwidget*)((WDragObject*)Source)->wi;
  if (wi) {
    NameEdit->Text=wi->getsymbol()->getfullnametext();
    if (((WDragObject*)Source)->todestroy) delete wi;
  }
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::HideToolbar1Click(TObject *Sender)
{
  HideToolbar1->Tag=1-HideToolbar1->Tag;
  if (HideToolbar1->Tag) {
    HideToolbar1->Caption="Show Toolbar";
    Panel1->Visible=false;
  } else {
    HideToolbar1->Caption="Hide Toolbar";
    Panel1->Visible=true;
  }
  OnResize(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TWindowForm::NormalClick(TObject *Sender)
{
  Normal->Checked=true;
  Refreshspeed=1;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::FastClick(TObject *Sender)
{
  Fast->Checked=true;
  Refreshspeed=2;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::NorefreshClick(TObject *Sender)
{
  Norefresh->Checked=true;
  Refreshspeed=3;
}
//---------------------------------------------------------------------------


void __fastcall TWindowForm::Run1Click(TObject *Sender)
{
  UpdateSpeed();
  if (Project.RunMode==RUN_STOP)
  switch (Refreshspeed) {
    case 1:
      Project.RunMode=RUN_ANIM|RUN_TICK | RUN_NORMANIM;
      while (Simulate(1)>=0);
      break;
    case 2:
      Project.RunMode=RUN_ANIM|RUN_TICK | RUN_FASTANIM;
      while (Simulate(1)>=0);
      break;
    case 3:
      Project.RunMode=RUN_GO;
      while (Simulate(1)>=0);
      break;
  }
}

void __fastcall TWindowForm::Step1Click(TObject *Sender)
{
  if (Project.RunMode==RUN_STOP)
  {
    Project.RunMode=RUN_ANIM|RUN_STEP|RUN_ONCE;
    while (Simulate(1)>=0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Trace1Click(TObject *Sender)
{
  if (Project.RunMode==RUN_STOP)
  {
    Project.RunMode=RUN_ANIM|RUN_TRACE|RUN_ONCE;
    while (Simulate(1)>=0);
  }
}

void __fastcall TWindowForm::Ticks1Click(TObject *Sender)
{
  if (Project.RunMode==RUN_STOP)
  {
    Project.RunMode=RUN_ANIM|RUN_TICK|RUN_ONCE;
    while (Simulate(0)>=0);
  }
}
//---------------------------------------------------------------------------


void __fastcall TWindowForm::Reset1Click(TObject *Sender)
{
  Project.Reset();
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Break1Click(TObject *Sender)
{
  Project.RunMode=RUN_STOP;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Bytrace1Click(TObject *Sender)
{
  Project.RunMode=RUN_ANIM|RUN_SLOWANIM|RUN_TRACE;
  Form1->Timer2->Enabled=true;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Bystep1Click(TObject *Sender)
{
  Project.RunMode=RUN_ANIM|RUN_SLOWANIM|RUN_STEP;
  Form1->Timer2->Enabled=true;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Byticks1Click(TObject *Sender)
{
  Project.RunMode=RUN_ANIM|RUN_SLOWANIM|RUN_TICK;
  Form1->Timer2->Enabled=true;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Animatemodes1Click(TObject *Sender)
{
  UpdateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Action=caFree;
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::Setfastanimationspeed1Click(TObject *Sender)
{
  AnimSpeedDialog->SlowSpeed->Value=Form1->Timer2->Interval;
  AnimSpeedDialog->HighSpeed->Value=Project.FastAnimateSpeed;
  if (AnimSpeedDialog->ShowModal()) {
    Form1->Timer2->Interval=AnimSpeedDialog->SlowSpeed->Value;
    Project.FastAnimateSpeed=AnimSpeedDialog->HighSpeed->Value;
  }
}
//---------------------------------------------------------------------------

void __fastcall TWindowForm::FormDestroy(TObject *Sender)
{
  if (!closing) {
    closing=1;
    delete win;
    win=NULL;
  }
}
//---------------------------------------------------------------------------

