/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __ARRAY_H__
#define __ARRAY_H__

class array : public symbol
{
  char *ptr;
  int startaddr;
  int endaddr; // endaddr is excluded!!!
public:
  virtual symbolwidget *getwidget() { return new arraywidget(getattributes(), this); }
  int cangetvalue(int addr,int size) { if (addr<startaddr) return 0;
    if (addr>endaddr-size) return 0; return 1;};
  unsigned char getcharvalue(int addr) { if (cangetvalue(addr,1)) return ptr[addr]; else return 0; }
  void setcharvalue(int addr, char value) { if (cangetvalue(addr,1)) ptr[addr] = value; }
  word getwordvalue(int addr) { if (cangetvalue(addr,2)) return *((word*)&ptr[addr]); else return 0;}
  void setwordvalue(int addr, char value) { if (cangetvalue(addr,2)) *((word*)&ptr[addr]) = value;}
  unsigned int  getdwordvalue(int addr) { if (cangetvalue(addr,4)) return *((int*)&ptr[addr]); else return 0;}
  void setdwordvalue(int addr, char value) { if (cangetvalue(addr,4)) *((int*)&ptr[addr]) = value;}
  void getmemrange(int *start, int *end)
    { if (start != NULL) *start = startaddr; if (end != NULL) *end = endaddr; }
  array(int name, attribute *attribs, symbol *parent, char *ptr, int s, int e) : symbol(name, attribs, parent)
    { startaddr = s; endaddr = e; array::ptr=ptr; }
  symbol *_getlocalsymbol(symbolname *s)
    {
      if (*s->elem >= 0 && *s->elem >= startaddr && *s->elem < endaddr)
	return new basevariable<char>(*s->elem, getattributes(), this, ptr + *s->elem);

      char *str = symbtable.tostring(*s->elem);
      if (str != NULL && str[0] == 'w')
      { 
	int addr = atoi(str+1);
        if (addr >= startaddr && addr < endaddr)
        {
	 
	  if (addr >= endaddr) return new basevariable<unsigned char>(*s->elem, getattributes(), this, (unsigned char *) (ptr + addr));
	  else return new basevariable<unsigned short>(*s->elem, getattributes(), this, (unsigned short *) (ptr + addr));
	}
      }

      return (symbol *) NULL;
    }
};

#endif
