/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

component::component(componenttype *type, int name, attribute *attribs, symbol *parent) : symbol(name, attribs, parent)
{ 
  SetUpdateNeeded();
  component::type = type; 
  //  DEBUG(cerr << "New component of type " << symbtable.tostring(type->name) << "\n";) 
}

void component::Reset()
{
  SetUpdateNeeded();
  updatetime = +INFINITY;
  _Reset();
}

void component::SetUpdateNeeded() 
{
  //  DEBUG(cerr << "  Set update needed " << getfullnametext() << "\n";)
  updateneeded = 1; 
  if (getparent()->ToComponent() != NULL)
    getparent()->ToComponent()->SetUpdateNeeded();
}

macrocomponent::macrocomponent(componenttype *type, int nameid, attribute *attribs, symbol *parent) : component(type, nameid, attribs, parent)
{
  components = (componentlist *) NULL;
  nets = (netlist *) NULL;
  updateneeded = 1;
  updatetime = +INFINITY;
}

list<symbol> *macrocomponent::getapparentsymbols() 
{ 
  for (list<symbolconstructor> *l = ((macrocomponenttype *) type)->symbolconstructors; l != NULL; l = l-> next)
  {
    int i = l->elem->getname();
    list<int> n(&i);
    getsymbol(&n);
  }

  return children; 
}

void macrocomponent::_Reset()
{
  for (list<component> *l = components; l != NULL; l = l->next)
    l->elem->Reset();
}

symbol *macrocomponent::_getlocalsymbol(symbolname *s)
{
  for (list<symbolconstructor> *l = ((macrocomponenttype *) type)->symbolconstructors; l != NULL; l = l-> next)
    if (l->elem->getname() == *(s->elem)) 
    {
      symbol *sy = l->elem->create(this);
      addsymbol(sy);
      return sy->getlocalsymbol(s->next);
    }

  return (symbol *) NULL;
}

void macrocomponent::Update(int force)
{ 
  //  cerr << "*Update component : " << symbtable.tostring(getname()) << "\n";

  updateneeded = 0; 
  updatetime = +INFINITY;

  for (netlist *nl = nets; nl != NULL; nl = nl->next)
    nl->elem->PropagateDown(force);
  for (componentlist *cl = components; cl != NULL; cl = cl->next) 
  { 
    //    cout << "*  " << cl->elem->getfullnametext() << "\n";

    if (Project.DispOptDebugInfo && cl->elem->UpdateNeeded())
      cout << "Update needed on " << cl->elem->getfullnametext() << "\n";

    if (force || cl->elem->UpdateNeeded() || Project.GetTime() >= cl->elem->UpdateTime())
      cl->elem->Update(force); 
    //   DEBUG(cerr << cl->elem->getfullnametext() << " : UT = " << cl->elem->UpdateTime() << ", UF = " << cl->elem->UpdateNeeded() << "\n";)
    updatetime = MIN(updatetime, cl->elem->UpdateTime());
  } 
  for (netlist *nl = nets; nl != NULL; nl = nl->next) 
    nl->elem->PropagateUp(); 
  for (componentlist *cl = components; cl != NULL; cl = cl->next) 
  { 
    if (!cl->elem->UpdateNeeded()) continue;
    //    cerr << cl->elem->getfullnametext() << "\n";
    updateneeded = 1;
    break;
  } 
  //  DEBUG(cerr << "selected : " << getfullnametext() << " : UT = " << UpdateTime() << ", UF = " << UpdateNeeded() << "\n";)
} 

componentconstructor::componentconstructor(int name, attribute *attribs, macrocomponenttype *parent) 
: symbolconstructor(name, attribs) 
{ 
  componentconstructor::parent = parent;
  attribs = nullattribute;
}

symbol *componentconstructor::_ccreate(macrocomponent *parent)
{
  component *c = type->create(name, attribs, parent);

  if (c == NULL)
  {
    createerror(TYPE_ERROR, "Unable to create component %s", symbtable.tostring(name));
    return (symbol *) NULL;
  }

  parent->components = new componentlist(c, parent->components);
  return c;
} 
