/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __COMPONENT_TYPE_H__
#define __COMPONENT_TYPE_H__

#include "project.h"

class component;
class macrocomponenttype;
class componentconstructor;
class symbolconstructor;

class componenttype
{
public:
  componenttype();
  int name;
  virtual component *create(int name, attribute *attrib, symbol *parent) = 0;
};

class macrocomponenttype : public componenttype
{
  attribute *attribs;
  friend componentconstructor;
  friend macrocomponent;
  friend int yyparse();
  list<symbolconstructor> *symbolconstructors;

public:
  macrocomponenttype();
  virtual ~macrocomponenttype();

  component *create(int name, attribute *attributes, symbol *parent);

  void AddAttributes(attribute *att) { attribs = att->addattribstoend(attribs); };
  void AddSymbol(symbolconstructor *s);
  //  void AddNet(int NetName, list<symbolname> *pins, attribute *attribs);
  void AddComponent(int ComponentType, list<componentconstructor> *ccs);
};

template <class t> class genericcomponenttype : public componenttype
{
public:
  component *create(int name, attribute *attribs, symbol *parent) {
    return new t(this, name, attribs, parent);
  }
};

void RegisterComponents();

#endif

