/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __LIST_H
#define __LIST_H

template <class t> class list
{
public:
  t *elem;
  list *next;

  t *getlast();
  int isprefix(list *l);
  list(t *elem, list *list = (list *) NULL);  //MODIFIED BY FM
  ~list() {
    if (next != NULL) delete next;
   }
};

template <class t> list<t>::list(t *elem, list *next)
{
  list::next = next;
  list::elem = elem;
}

typedef list<int> intlist;

template <class t> void revlist(list<t> *l)
{
  list<t> *cur, *prev, *next;
  t *oldelem, *curelem;

  if (l == NULL || l->next == NULL) return;
  
  cur = l->next;
  prev = (list<t> *) NULL;
  oldelem = l->elem;

  for (; cur != NULL; cur = next)
  {
    next = cur->next;
    cur->next = prev;
    curelem = cur->elem;
    cur->elem = oldelem;
    oldelem = curelem;
    prev = cur;
  }
  
  l->next = prev;
  l->elem = oldelem;
}

template <class t> list<t> *addtoend(list<t> *l, t *nelem)
{
  list<t> *nl = new list<t>(nelem, (list<t> *) NULL);

  return addtoend(l, nl);
}

template <class t> list<t> *addtoend(list<t> *l, list<t> *nl)
{
  if (l == NULL) return nl;

  list<t> *tl;
  for (tl = l; tl->next != NULL; tl = tl->next);
  tl->next = nl;
  return l;
}

template <class t> int list<t>::isprefix(list *l)
{
  for (list<t> *l2 = this; l != NULL; l = l->next, l2 = l2->next)
  {
    if (l2 == NULL || *(l->elem) != *(l2->elem)) return 0;
  }

  return 1;
}

template <class t> t *list<t>::getlast()
{
  // Can't be called on null list

  list<t> *l;
  for (l = this; l->next != NULL; l = l->next);
  
  return l->elem;
}

#endif


