/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "project.h"

symbolwidget *variable::getwidget()
{
  return new variablewidget(getattributes(), this);
}

int variable::setvalue(char *strvalue, int defbase)
{
  unsigned long value=0;
  if (::setstringvalue(strvalue,defbase,&value)==0) setvalue(value);
  return 0;
}

char *variable::getstringvalue(int base)
{
  return ::getstringvalue(getvalue(), getsize(), base);
}

int setstringvalue(char *strvalue, int defbase, unsigned long* result)
{
  unsigned long value=0;
  int newbase=defbase;
  //skip spaces
  while (*strvalue==32) strvalue++;
  //search base information
  int i=strlen(strvalue);
  int j=0;
  if ((i)&&(strvalue[0]=='$')) { newbase=16;j=1; }
  if ((i>=2)&&(strvalue[0]=='0')&&((strvalue[1]=='x')||(strvalue[1]=='X'))) { newbase=16;j=2; }
  if (!j) {
    bool cont=true;
    bool spacefound=false;
    i=0;
    while (cont) {
    //decode base information
      if ((strvalue[i]<'0')||(strvalue[i]>'9'))
      switch (strvalue[i]) {
        case 'h': case 'H':{
          newbase=16;
          if (spacefound) {
            cont=false;defbase=newbase;
          }
          break;
        }
        case 'o': case 'O':{
          newbase=8;
          if (spacefound) {
            cont=false;defbase=newbase;
          }
          break;
        }
        case 'b': case 'B':{
          newbase=2;
          if (spacefound) {
            cont=false;defbase=newbase;
          }
          break;
        }
        case 'd': case 'D':{
          newbase=10;
          if (spacefound) {
            cont=false;defbase=newbase;
          }
          break;
        }
        case '_':{
          //find base number
          if ((strvalue[i]>='0')&&(strvalue[i]<='9')) {
            newbase=0;
            for (i++;(strvalue[i]>='0')&&(strvalue[i]<='9');i++) {
                newbase=newbase*10+strvalue[i]-'0';
            }
            if (newbase<=1) return -1;
            defbase=newbase;
            cont=false;
          } else spacefound=true;
          break;
        }
        case ' ':{
          spacefound=true;
          break;
        }
        case 0: {cont=false;break;}
        case '-' : case 'A': case 'a': case 'C' : case 'c' :
        case 'E' : case 'e': case 'F': case 'f' : case '!' :
        case '~' : break;
        default: {
          //error invalid string
          return -1;
        }
      }  //switch
      i++;
    } // while
    if ((newbase==2)&&(defbase>'B'-'A'+10)) newbase=defbase;
    if ((newbase==8)&&(defbase>'O'-'A'+10)) newbase=defbase;
    if ((newbase==10)&&(defbase>'D'-'A'+10)) newbase=defbase;
    if ((newbase==16)&&(defbase>'H'-'A'+10)) newbase=defbase;
  }
  //skip spaces
  while (strvalue[j]==32) j++;
  //gestion de signe et inversion
  char sign=strvalue[j];
  if ((sign!='-')&&(sign!='!')&&(sign!='~')) sign=0;
    else j++;
  //read number
  int val=0;
  while ((val>=0)&&(val<newbase)) {
    val=-1;
    if ((strvalue[j]>='0')&&(strvalue[j]<='9')) val=strvalue[j]-'0';
    if ((strvalue[j]>='A')&&(strvalue[j]<='Z')) val=strvalue[j]-'A'+10;
    if ((strvalue[j]>='a')&&(strvalue[j]<='z')) val=strvalue[j]-'a'+10;
    if ((val>=0)&&(val<newbase)) value=value*newbase+val;
      else {
        if ((val>=0)&&(val<=9)) return -1;
        //find errors
        switch (strvalue[j]) {
          case 'B': case 'b' : { if (newbase!=2) return -1; break;}
          case 'O': case 'o' : { if (newbase!=8) return -1; break;}
          case 'D': case 'd' : { if (newbase!=10) return -1; break;}
          case 'H': case 'h' : { if (newbase!=16) return -1; break;}
          case ' ': case 0 : case '_' : break;
          default: return -1;
        }
      }
    j++;
  }
  switch (sign) {
    case '!': case '~' : { value=~value;break;}
    case '-':{ value=-value;break;}
  }
  if (result) *result=value;
  return 0;
}


char *getstringvalue(unsigned int value, int size, int base)
{
  if (base > 36) base = 16;
  if (base <= 1) base = 16;

  static char buff[34];

  int i;
  int numbits = size;
  unsigned long bitfield = ~((~0) << numbits);
  for (i = 0; bitfield != 0; i++) // Find the number of digits to display.
    bitfield /= base;
  switch(base)
  {
    case 2:
      buff[i] = 'b';
      buff[i+1] = 0;
      break;
    case 8:
      buff[i] = 'o';
      buff[i+1] = 0;
      break;
    case 10:
      buff[i] = 'd';
      buff[i+1] = 0;
      break;
    case 16:
      buff[i] = 'h';
      buff[i+1] = 0;
      break;
    default:
      buff[i] = '_';
      if (base >= 10) buff[i+1] = '0' + base / 10, buff[i+2] = '0' + base % 10, buff[i+3] = 0;
      if (base < 10) buff[i+1] = '0' + base, buff[i+2] = 0;
  }
  unsigned int val = value;
  for (i--; i >= 0; i--, val /= base)
  {
    int digit = val % base;
    buff[i] = (digit < 10 ? '0' : 'A' - 10) + digit;
  }

  return buff;
}

char *variable::getsignedstringvalue(int base)
{
  return ::getsignedstringvalue(getsignedvalue(), getsize(), base); 
}

char *getsignedstringvalue(int value, int size, int base)
{
  if (base > 36) base = 16;
  if (base <= 1) base = 16;

  static char buff[35];

  int i;
  int numbits = size;
  unsigned long bitfield = ~((~0) << (numbits - 1)) + 1;
  for (i = 1; bitfield != 0; i++) // Find the number of digits to display.
    bitfield /= base;
  switch(base)
  {
    case 2:
      buff[i] = 'b';
      buff[i+1] = 0;
      break;
    case 8:
      buff[i] = 'o';
      buff[i+1] = 0;
      break;
    case 10:
      buff[i] = 'd';
      buff[i+1] = 0;
      break;
    case 16:
      buff[i] = 'h';
      buff[i+1] = 0;
      break;
    default:
      buff[i] = '_';
      if (base >= 10) buff[i+1] = '0' + base / 10, buff[i+2] = '0' + base % 10, buff[i+3] = 0;
      if (base < 10) buff[i+1] = base, buff[i+2] = 0;
  }
  int val = value;
  buff[0] = val > 0 ? '+' : (val < 0 ? '-' : ' ');
  if (val < 0) val = -val;
  for (i--; i > 0; i--, val /= base)
  {
    int digit = val % base;
    buff[i] = (digit < 10 ? '0' : 'A' - 10) + digit;
  }

  return buff;
}

unsigned long fusionvariable::getvalue()
{
  long retval = 0;

  for (list <fusionvariableelement> *l = vars; l != NULL; l = l->next)
  {
    variable *v = l->elem->var;
    retval = (retval << (1 + l->elem->end - l->elem->start)) + v->getbits(l->elem->start, l->elem->end);
  }

  return retval;
}

void fusionvariable::setbits(int start, int end, unsigned long val)
{
  int curpos = bits;

  for (list <fusionvariableelement> *l = vars; l != NULL; l = l->next)
  {
    curpos -= l->elem->end - l->elem->start + 1;

    if (curpos + l->elem->end + 1 - l->elem->start < start) break;
    if (curpos > end) continue;
    int srcstart = MAX(0, curpos - start);
    int deststart = l->elem->start + start - curpos + srcstart;
    int srcend = MIN(end - start, curpos - start + l->elem->end - l->elem->start);
    int destend = deststart + srcend - srcstart;

    l->elem->var->setbits(deststart, destend, (~((~1) << srcend) & val) >> srcstart);
  }    
}

fusionvariable::fusionvariable(int name, attribute *attribs, symbol *parent, list<fusionvariableelement> *vars) : variable(name, attribs, parent) 
{ 
  fusionvariable::vars = vars; 
  bits = 0;
  for (list<fusionvariableelement> *l = vars; l != NULL; l = l->next)
    bits += 1 + l->elem->end - l->elem->start;
}

template <class t> void basevariable<t>::setbits(int start, int end, unsigned long val)
{
 unsigned long mask = ((~1) << end) | ~((~0) << start);
 *ptr = ((val << start) & ~mask) | (*ptr & mask);
}

symbol *variableconstructor::_create(symbol *parent)
{
  list<fusionvariableelement> *prod = (list<fusionvariableelement> *) NULL;
  fusionvariableelement *f;

  for (list<symbolname> *l = pieces; l != NULL; l = l->next)
  {
    symbol *s = parent->getsymbol(l->elem);

    if (s != NULL) 
    {
      variable *v = s->ToVariable();
      if (v != NULL)
      {
	f = new fusionvariableelement;
	f->var = v;
	f->start = 0;
	f->end = v->getsize() - 1;
      }
      else 
      {
	createerror(TYPE_ERROR, "Not a variable %s while creating variable %s", symbtable.tostring(l->elem), symbtable.tostring(name));
	continue;
      }
    }
    else
    { // We may still have a bit from a variable.
      if (l->elem == NULL) 
      {
        createerror(TYPE_ERROR, "Internal error : variableconstructor::_create should not recieve __emptyname__");
	continue;
      }
      symbol *s = parent->getparent(l->elem);
      if (s == NULL) 
      {
	createerror(TYPE_ERROR, "Can't find %s while creating %s", symbtable.tostring(l->elem), symbtable.tostring(name));
	continue;
      }
      variable *v = s->ToVariable();
      if (v == NULL) 
      {
	createerror(TYPE_ERROR, "Not a variable %s while creating variable %s", symbtable.tostring(l->elem), symbtable.tostring(name));
	continue;
      }
      
      int start = *(l->elem->getlast()); // find all the bits that we can group together.
      int end = start;
      if (v->getsize() <= end) 
      {
	createerror(TYPE_ERROR, "Bad subscript %i in variable %s", end, v->getfullnametext());
	continue;
      }
      
      while (1) 
      { // Combine if
	if (l->next == NULL) break;// More to combine with
	symbolname *sn = l->next->elem;

	s = parent->getparent(sn); // Next name gives me a pointer to the same variable
	if (s == NULL) break;
	variable *v2 = s->ToVariable();
	if (v2 == NULL) break;
	if (v != v2) break;

	if (*(sn->getlast()) != end + 1) break;// Consecutive with previous bits

	if (v2->getsize() <= end + 1) 
	  createerror(TYPE_ERROR, "Bad subscript %i in variable %s", end + 1, v2->getfullnametext());

	end++;
	l=l->next;
      }
      f = new fusionvariableelement;
      f->var = v;
      f->start = start;
      f->end = end;  
    }
    
    prod = new list<fusionvariableelement>(f, prod);
  }
  
  return new fusionvariable(name, attribs, parent, prod);
}

/*unsigned long getbits(int start, int end)
{
  int curbit = 0;
  list <fusionvariableelement> *l = vars;
  
  for (; l != NULL && curbit + l->elem->end - l->elem->start < start; l = l->next) 
    curbit += l->elem->end - l->elem->start + 1;

  if (l == NULL) return 0;
  if (l->elem->end - l->elem->start >= end - start)
    return l->elem->var->getbits(l->elem->start + start - curbits, l->elem->start + end - curbits);

  unsigned long retval = l->elem->var->getbits(l->elem->start + start - curbits, l->elem->end);
;
(....)
  for (list <fusionvariableelement> *l = vars; l != NULL; l = l->next)
  {
    variable *v = l->elem->var;
    retval = (retval << v->getsize()) + v->getbits(l->elem.start, l->elem.end);
  }
}
*/
