/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __WIDGET_H__
#define __WIDGET_H__

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

class window;
class symbolwidget;
class symbolborderwidget;
class boxwidget;

extern int idletype;
extern gint idletag;

#define IDLE_NONE 0
#define IDLE_IDLE 1
#define IDLE_DELAY 2

void PrepareStyles();

int setidletimeout(gint time, int keep = 0);

inline void QuitProgram() 
{
  gtk_main_quit(); 
}

class widget
{
  friend symbolborderwidget;

  list<attribute> *attribbacklog;
  int startup :1;

protected:
  attribute *initattribs;
  GtkWidget *wi;
  virtual void draw();
  void passtochildren(void (widget::*fn)()) { passtochildren((void (widget::*)(void *)) fn, NULL); };
  virtual void passtochildren(void (widget::*fn)(void *), void *p);
  void refreshwindow();
  virtual void _editmode() {}
  virtual void _runmode() {}
  virtual void _parseattribs(attribute *attribs);

public:
  void localparseattribs(attribute *attribs);
  void editmode() { _editmode(); passtochildren(&widget::editmode); }
  void runmode() { _runmode(); passtochildren(&widget::runmode); }
  virtual void RemoveChild(symbolborderwidget *sbw) { error(TYPE_INTERNALERROR, -1, NULL, "Internal error : widget::RemoveChildWidget() called!\n"); };
  void parseattribs(attribute *attribs);
  void display();
  void refresh();
  widget(attribute *attribs);
  virtual ~widget();
  void RemoveFromContainer(GtkWidget *c);
  virtual GtkWidget *GetWindowWidget() { return (GtkWidget *) NULL; }

  virtual void export(int indent) = 0;
  virtual void exportattribs(int putcomma) {}
};

class window : public widget
{
   GtkWidget *box;
   GtkWidget *editbox;
   GtkWidget *runbox;
   symbolborderwidget *w;

   static void reset();
   static void delete_event(GtkWidget *widget, GdkEvent  *event, window *_this);
   static void newwindow(GtkButton *button, int type);
   static void getboxdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, window *_this);
   static void trashusedropdata(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  gpointer udata);
   static void newwindowfromselection(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  gpointer udata);
   static void runmodeclicked(GtkButton *button, window *_this) { _this->runmode(); };
   static void editmodeclicked(GtkButton *button, window *_this) { _this->editmode(); };
   static void setrunmode(GtkButton *button, int mode);
   static void exportsig(GtkButton *button, window *_this);
   static void droptoeditzone(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  gpointer udata);
   static void editzonehighlight(GtkWidget *wid);
   static void completeeditzone(GtkWidget *wid);

   unsigned int destroying : 1;

   void construct(symbolborderwidget *sw);

   int OldRunMode;

protected:

   virtual void _editmode();
   virtual void _runmode();
   void draw();
   void passtochildren(void (widget::*fn)(void *), void *);

public:
   ~window();
   void RemoveChild(symbolborderwidget *sbw);
   window(attribute *attribs, symbolwidget *);
   window(attribute *attribs, symbolborderwidget *);
   GtkWidget *GetWindowWidget() { return wi; }
   virtual void export(int indent);
};

class symbolborderwidget : public widget
{
  static GtkItemFactoryEntry PopupMenu[];
  static GtkItemFactory *CreatePopupMenuFactory(GtkAccelGroup **accel);
  static GtkItemFactory *GetPopupMenuFactory(GtkAccelGroup **accel = (GtkAccelGroup **) NULL);
  static symbolborderwidget *popupmenuwidget;

  GtkWidget *AssociatedSeparator;
  widget *parent;
  void settitle();
  int extname;
  
  struct 
  {
    unsigned int extname : 1;
    unsigned int title : 1;
    unsigned int full : 1;
    unsigned int hasborder : 1;
  } displaymode;
  symbolwidget *s;
  GtkWidget *title;
  GtkWidget *border;
  GtkWidget *box;

  static void getdropdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, symbolborderwidget *_this);
protected:
  static void begindrag(GtkWidget *widget, GdkDragContext *drag_context, symbolborderwidget *_this);
  static void enddrag(GtkWidget *widget, GdkDragContext *drag_context, symbolborderwidget *_this);
  //  static void disactivate(GtkWidget *widget, GdkEventCrossing *event, symbolborderwidget *_this);
  static void activate(GtkWidget *widget, GdkEventCrossing *event, symbolborderwidget *_this);
  static GtkWidget *Activated;
  static gboolean popupmenu(GtkWidget *widget, GdkEventButton *event, symbolborderwidget *_this);
  //  static void setbase(gpointer callback_data, guint base, GtkWidget *widget);
  static void menusetattribute(gpointer callback_data, attribute *attribs, GtkWidget *widget);
  static void menusetattributelocal(gpointer callback_data, attribute *attribs, GtkWidget *widget);
  static void setrunmode(gpointer callback_data, int mode, GtkWidget *widget);
  virtual void passtochildren(void (widget::*fn)(void *), void *);
  virtual void _parseattribs(attribute *attribs);
public:
  symbol *getsymbol();
  int canexpand() { return displaymode.hasborder; }
  void setname(int id) { extname = id; displaymode.extname = 1; settitle(); };
  GtkWidget *GrabGtkWidget(widget *p, GtkWidget *AssocSep);
  symbolborderwidget(attribute *attribs, symbolwidget *s);
  //  void RemoveFromParent() { parent->RemoveChild(this); }
  GtkWidget *GetAssociatedSeparator() { return AssociatedSeparator; }
  ~symbolborderwidget();
  widget *getparent() { return parent; }
  static GtkAccelGroup *GetPopupMenuAccelGroup();

  virtual void export(int indent);
  virtual void exportattribs(int putcomma);
};

class symbolwidget : public widget
{
  friend class plugwidget; // horrible!!!
  symbol *s;
  symbolborderwidget *parent;

protected:

public:
  GtkWidget * GetGtkWidget(symbolborderwidget *p) { parent = p; return wi; }
  symbol *getsymbol() { return s; }
  symbolwidget(attribute *attribs, symbol *s);
  virtual int hasborder() { return 0; }

  GtkWidget *GetWidget() { return wi; }
  virtual void export(int indent);
  virtual int exportattribs() { return 0; };
};

// For development purposes only.
class plainsymbolwidget : public symbolwidget
{
public:
  plainsymbolwidget(attribute *attribs, symbol *s);
};

class boxwidget : public symbolwidget
{
  static void usedropdata(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  boxwidget *_this);

  GtkWidget *(*new_separator_func)();
  GtkWidget *newseparator();

  void initdrop(GtkWidget *w);

protected:
  list<symbolborderwidget> *widgets;
  GtkWidget *lastseparator;
  void insertwidget(symbolborderwidget *symb, GtkWidget *psep);
  virtual void passtochildren(void (widget::*fn)(void *), void *);

public:
  virtual int hasborder() { return 1; }
  void RemoveChild(symbolborderwidget *sbw);
  boxwidget(attribute *attribs, displaybox *s, GtkBox *box, GtkWidget *(*makeseparator)());

  virtual char *exporttype() = 0;
  virtual void export(int indent);
};

class hboxwidget : public boxwidget
{
public:
  hboxwidget(attribute *attribs, displaybox *s);
  virtual char *exporttype() { return "hbox"; }
};

class vboxwidget : public boxwidget
{
protected:
   virtual void _editmode();
   virtual void _runmode();
public:
  vboxwidget(attribute *attribs, displaybox *s);

  virtual char *exporttype() { return "vbox"; }
};

/*class consttextwidget : public symbolwidget
{
protected:
public:
  consttextwidget(attribute *attribs, char *text);
  };*/

class pinwidget : public symbolwidget
{
protected:
  void draw();
public:
  pinwidget(attribute *attribs, pin *p);
};

class variablewidget : public symbolwidget
{
  int oldvaluestate;
  unsigned long oldvalue;
  real oldvaluetime;

  int base;
  unsigned int usesigned : 1;
  static void edited(GtkEntry *w, variablewidget *_this);
  static void edited2(GtkEntry *w, GdkEventFocus *event, variablewidget *_this);
  //  static gboolean keypress(GtkWidget *widget, GdkEventKey *event, variablewidget *_this);
  //  static gboolean clicked(GtkWidget *widget, GdkEventButton *event, variablewidget *_this);
  int height;
  int width;

protected:
  void draw();
  virtual void _parseattribs(attribute *attribs);
public:
  variablewidget(attribute *attribs, variable *var);
  virtual int exportattribs();
};

class realwidget : public symbolwidget
{
  int oldvaluestate;
  real oldvalue;
  real oldvaluetime;

  int base;
  unsigned int usesigned : 1;
  static void edited(GtkEntry *w, variablewidget *_this);
  static void edited2(GtkEntry *w, GdkEventFocus *event, variablewidget *_this);
  int height;
  int width;

protected:
  void draw();
public:
  realwidget(attribute *attribs, realsymbol *r);
};

class plugwidget : public vboxwidget
{
public:
  plugwidget(attribute *attribs, plug *p);
};

class disassemblywidget : public symbolwidget
{
  int OldIP;
  int saddr, eaddr;
  void draw();
  GtkWidget *listwid;
  char *BreakState;

  int selectcurline();
  void jumptoline(int row = -1);
  real prevtime;
  static void selectrow(GtkCList *clist, gint row, gint column, GdkEventButton *event, disassemblywidget *_this);
  int getfocusaddress();
  int minheight;
protected:
  virtual void _parseattribs(attribute *attribs);
public:
  virtual int hasborder() { return 1; }
  ~disassemblywidget();
  disassemblywidget(attribute *attribs, disassembly *d);
  virtual int exportattribs();
};

class oscillowidget : public symbolwidget
{
  int width, height;

  void draw();

  static void drawevent(GtkWidget *widget, GdkEventExpose *event, oscillowidget *_this);
  static void configureevent(GtkWidget *widget, GdkEventConfigure *event, oscillowidget *_this);
public:
  virtual int hasborder() { return 1; }
  oscillowidget(attribute *attribs, Oscillo *o);
};

class projectwidget : public symbolwidget
{
  int displayanonymous;

  //  GtkCTreeNode *SelectedRow;
  GtkWidget *treewid;
  GtkCTreeNode *insertnode(GtkCTreeNode *parent, symbol *s);
  void createchildren(GtkCTreeNode *ctnode);

  static void createchildren(GtkCTree *ctree, GtkCTreeNode *node, projectwidget *_this);
  static void expand(GtkCTree *ctree, GList *node, projectwidget *_this);
  static void getdropdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, projectwidget *_this);
  int minheight;

protected:
  virtual void _parseattribs(attribute *attribs);
public:
  virtual int hasborder() { return 1; }
  projectwidget(attribute *attribs, project *proj);
  virtual int exportattribs();
};

class arraywidget : public symbolwidget
{
  unsigned int wordsize : 1;
  int w, h;
  int start, end;
  GtkWidget *table;
  symbolwidget **cells;
  symbolwidget **cellsw;
  void RebuildArray();
protected:
  virtual void passtochildren(void (widget::*fn)(void *), void *);
  virtual void _parseattribs(attribute *attribs);
public:
  //  virtual int hasborder() { return 1; }
  ~arraywidget();
  arraywidget(attribute *attribs, array *a);
  virtual int exportattribs();
};

#endif
