/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __WIDGET_H__
#define __WIDGET_H__

#include <gtk/gtk.h>

class window;
class symbolwidget;
class symbolborderwidget;
class boxwidget;

inline void QuitProgram() 
{
  gtk_main_quit(); 
}

class widget
{
  int startup :1;
protected:
  attribute *initattribs;
  GtkWidget *wi;
  virtual void draw();
  virtual void passtochildren(void (widget::*fn)());
  void refreshwindow();
public:
  virtual void RemoveChild(symbolborderwidget *sbw) { cerr << "Internal error : widget::RemoveChildWidget() called!\n"; exit(1); };
  virtual void parseattribs(attribute *attribs) {};
  void display();
  void refresh();
  widget(attribute *attribs);
  virtual ~widget();
  void RemoveFromContainer(GtkWidget *c);
  virtual GtkWidget *GetWindowWidget() { return (GtkWidget *) NULL; }
};

class window : public widget
{
   GtkWidget *box;
   symbolborderwidget *w;

   static void delete_event(GtkWidget *widget, GdkEvent  *event, window *_this);
   static void newwindow(GtkButton *button, int type);
   static void getboxdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, window *_this);

   int destroying : 1;
protected:
   static void trashusedropdata(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  gpointer udata);

   void draw();
   void passtochildren(void (widget::*fn)());
public:
   ~window();
   void RemoveChild(symbolborderwidget *sbw);
   window(attribute *attribs, symbolwidget *);
   GtkWidget *GetWindowWidget() { return wi; }
};

class symbolborderwidget : public widget
{
  GtkWidget *AssociatedSeparator;
  widget *parent;
  void settitle();
  struct 
  {
    int title :1;
    int full :1;
    int hasborder :1;
  } displaymode;
  symbolwidget *s;
  GtkWidget *title;
  GtkWidget *border;
  static void getdropdata(GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time, symbolborderwidget *_this);
protected:
  static void symbolborderwidget::begindrag(GtkWidget *widget, GdkDragContext *drag_context, symbolborderwidget *_this);
  static void enddrag(GtkWidget *widget, GdkDragContext *drag_context, symbolborderwidget *_this);

  virtual void passtochildren(void (widget::*fn)());
public:
  GtkWidget *GrabGtkWidget(widget *p, GtkWidget *AssocSep);
  symbolborderwidget(attribute *attribs, symbolwidget *s);
  //  void RemoveFromParent() { parent->RemoveChild(this); }
  GtkWidget *GetAssociatedSeparator() { return AssociatedSeparator; }
  ~symbolborderwidget();
  widget *getparent() { return parent; }
};

class symbolwidget : public widget
{
  symbol *s;
  symbolborderwidget *parent;

protected:

public:
  GtkWidget * GetGtkWidget(symbolborderwidget *p) { parent = p; return wi; }
  symbol *getsymbol() { return s; }
  symbolwidget(attribute *attribs, symbol *s);
  virtual int hasborder() { return 0; }
};


// For development purposes only.
class plainsymbolwidget : public symbolwidget
{
public:
  plainsymbolwidget(attribute *attribs, symbol *s);
};

class boxwidget : public symbolwidget
{
  static void usedropdata(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time,  boxwidget *_this);

  GtkWidget *(*new_separator_func)();
  GtkWidget *newseparator();

  list<symbolborderwidget> *widgets;

  void initdrop(GtkWidget *w);
protected:
  void insertwidget(symbolborderwidget *symb, GtkWidget *psep);
  GtkWidget *lastseparator;
  virtual void passtochildren(void (widget::*fn)());
public:
  virtual int hasborder() { return 1; }
  void RemoveChild(symbolborderwidget *sbw);
  boxwidget(attribute *attribs, displaybox *s, GtkBox *box, GtkWidget *(*makeseparator)());
};

class hboxwidget : public boxwidget
{
public:
  hboxwidget(attribute *attribs, displaybox *s);
};

class vboxwidget : public boxwidget
{
public:
  vboxwidget(attribute *attribs, displaybox *s);
};

/*class consttextwidget : public symbolwidget
{
protected:
public:
  consttextwidget(attribute *attribs, char *text);
  };*/

class pinwidget : public symbolwidget
{
protected:
  void draw();
public:
  pinwidget(attribute *attribs, pin *p);
};

class variablewidget : public symbolwidget
{
 protected:
  void draw();
public:
  variablewidget(attribute *attribs, variable *var);
};

class plugwidget : public vboxwidget
{
public:
  plugwidget(attribute *attribs, plug *p);
};

/*class handlewidget : public symbolwidget
{
  symbolwidget *child;
  void draw();
  void passtochildren(void (widget::*fn)());
public:
  handlewidget(symbolwidget *c);
  };*/
#endif
