/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//---------------------------------------------------------------------------
#include "WindowFormUnit.h"
#include "fmcontrols.hpp"
#include "DisassemblyListUnit.hpp"
//#include "ChangeVarUnit.h"
#include "OscilloUnit.hpp"
#include "VariableUnit.hpp"
#include "ArrayUnit.hpp"
#include "RealUnit.hpp"
#include "BoxUnit.hpp"
#include "ProjectViewerUnit.hpp"
#include "clipbrd.hpp"
#include "PinViewer.hpp"

#define DEFINECONTROL(item) {control=item;\
                             (item)->OnMouseDown=MouseDown;\
                             (item)->OnDblClick=DblClick;}

#define PREPARE_DRAG_DROP(item) {(item)->OnStartDrag=StartDrag; \
                                 (item)->OnDragOver=DragOver;\
                                 (item)->OnDragDrop=DragDrop;\
                                 (item)->OnEndDrag=EndDrag;\
                                 (item)->DragMode=dmManual;}


/*class TSymbolNode : public TTreeNode {


};
 */
//****************************************************************************
TCursor dragcursor;


TCursor __fastcall WDragObject::GetDragCursor(bool Accepted, int X, int Y)
{
  if (Accepted) return dragcursor;
  else return crNoDrop;
}

void __fastcall WDragObject::Finished(System::TObject* Target, int X, int Y, bool Accepted)
{
  if ((!Accepted)&&(todestroy)) {
    //destroy widget
    delete wi;
  }
}

void QuitProgram()
{
}

/****************************************************************************
			       WIDGET
 ****************************************************************************/

widget::~widget()
{
  // #### Err!!
}

widget::widget(attribute *attribs)
{
  initattribs=attribs;
  parent=NULL;
}

void widget::refresh()
{
  draw();
  passtochildren(&symbolwidget::refresh);
}

void widget::parseattribs(attribute *attribs)
{
  localparseattribs(attribs);
  passtochildren((void (widget::*)(void *)) &widget::parseattribs, (void *) attribs);
}

/****************************************************************************
                   WINDOW
****************************************************************************/

window::window(attribute *attribs, symbolwidget *sw) : widget(attribs)
{
  TWindowForm *form;
  form=new TWindowForm(Application);
  ((TWindowForm*)form)->win=this;
  form->Show();
  form->OnResize=ContainerResize;
  form->Container->InsertControl(sw->GetVclWidget(this));
  Project.windows = new list<window>(this, Project.windows);

  control=form;
  window::sw=sw;
  parseattribs(attribs);
  ContainerResize(NULL);
  destroying=false;
}

void __fastcall window::ContainerResize(TObject *Sender)
{
  if ((!Application->Terminated)&&(!((TWindowForm*)control)->closing)&&(sw)) {
    TWindowForm* form=(TWindowForm*)control;
    int x,y,ox,oy;
    x=y=ox=oy=0;
    form->Container->Visible=false;
    sw->getsizewanted(x,y,ox,oy);
    sw->setsize(x,y);
    if (form->Autosize->Checked) {
      form->OnResize=NULL;
      int width=x+(form->Width-form->Container->Width+4);
      int height=y+(form->Height-form->Container->Height+4);
      form->Width=width;
      if (width<=128) form->Height=height+18; else form->Height=height;
      form->OnResize=ContainerResize;
    }
    form->Container->Visible=true;
    refresh();
  }
}

void window::updatewindow()
{
  ContainerResize(NULL);
}

window::~window()
{
  if (!destroying) {
    destroying=true;
    Project.UnlinkWindow(this);
    delete sw;
  }
}

void window::passtochildren(void (widget::*fn)())
{
  if (sw)
    (sw->*fn)();
}

void window::passtochildren(void (widget::*fn)(void *), void *p)
{
  if (sw)
    (sw->*fn)(p);
}

int window::RemoveChild(widget *w)
{
  if (w==sw) {
    ((TWindowForm*)control)->Container->RemoveControl(sw->GetVclWidget(this));
    sw=NULL;
    delete this;
    return 1;
  }
  return 0;
}

void window::exportwidget(TStream* stream,int indent)
{
  char buffer[200];
  TArrayInfo* arrayinfo=(TArrayInfo*)(control->Tag);
  sprintf(buffer,"width = %d; height = %d; left = %d; top = %d;",control->Width,control->Height,
  control->Left,control->Top);
  stream->Write(buffer,strlen(buffer));
  ((widget*)sw)->exportwidget(stream,indent);
}

void window::localparseattribs(attribute *attribs)
{
  int w;
  int updt;
  if (sw->getsymbol()->getattributes()->getintattribute("left", &w))
    control->Left=w;
  if (sw->getsymbol()->getattributes()->getintattribute("top", &w))
    control->Top=w;
}



/****************************************************************************
                   symbolwidget
****************************************************************************/


symbolwidget::symbolwidget(attribute *attribs, symbol *s) : widget(attribs)
{
  symbolwidget::sy = s;
  control=NULL;
  NameLength = 1;
  hasextname = 0;
}

void symbolwidget::getsizewanted(int& x,int& y,int& ox,int& oy)
{
  if (control) {
    control->Left=ox;
    control->Top=oy;
    y=MAX(y,oy+control->Height);
    x=MAX(x,ox+control->Width);
  }
}

void symbolwidget::setsize(int rx,int ry)
{
}

void __fastcall symbolwidget::StartDrag(System::TObject* Sender, TDragObject* &DragObject) {
  DragObject=new WDragObject();
  ((WDragObject*)DragObject)->wi=this;
  ((WDragObject*)DragObject)->todestroy=0;
}

void __fastcall symbolwidget::EndDrag(System::TObject* Sender, System::TObject* Target, int X, int Y) {
}

void __fastcall symbolwidget::DragOver(System::TObject* Sender, System::TObject* Source, int X, int Y, TDragState State, bool &Accept) {
  widget* wi=((WDragObject*)Source)->wi;
  Accept=(wi)&&(wi!=this)&&(!isparent(wi));
  dragcursor=crDrag;
}

void __fastcall symbolwidget::DragDrop(System::TObject* Sender, System::TObject* Source, int X, int Y) {
  //par dfaut: place l'objet  la suite du widget courant
  symbolwidget *wi=(symbolwidget*)((WDragObject*)Source)->wi;
  widget *wiparent=wi->parent;
  boxwidget *parentbox=(boxwidget*)parent;
  //Remove the widget from its owner
  int destroyed=1;
  if (wiparent)
    destroyed=wiparent->RemoveChild(wi);
  //Place it after the current item
  parentbox->InsertWidgetAfter(this,wi);
  //Update window
  updatewindow();
  if (!destroyed)
    if (wiparent->getparentwindow()!=getparentwindow()) wiparent->updatewindow();
}

void __fastcall symbolwidget::MouseDown(System::TObject* Sender, TMouseButton Button, Classes::TShiftState Shift, int X, int Y)
{
  if ((!Shift.Contains(ssDouble))&&(!Shift.Contains(ssRight)))
    (control)->BeginDrag(false);
}

char* __fastcall symbolwidget::OnGetName(int NameLength)
{
  //for debug pupose
  /*static char buffer[200];
  sprintf(buffer,"%0.8X %0.8X",this,parent);
  return buffer;
  */
  //
  variable *var = (variable *) getsymbol();
  if (hasextname) {
    if (NameLength) return symbtable.tostring(extname);
      else return "";
  }
  if (!(symbtable.isanonymous(sy->getname()))) {
    switch (NameLength) {
      case 0: {
        return "";
        break;
      }
      case 1: {
        return symbtable.tostring(sy->getname());
        break;
      }
      case 2: {
        return sy->getfullnametext();
        break;
      }
    }
  }
  return "";
}

void symbolwidget::exportwidget(TStream* stream,int indent)
{
  for (int i=0;i<indent;i++) stream->Write(" ",1);
  stream->Write("symbol ",7);
  stream->Write("(",1);
  exportattribs(stream);
  stream->Write(")",1);
  char* str=sy->getfullnametext();
  stream->Write(str,strlen(str));
  stream->Write(";\r\n",3);
}

void symbolwidget::localparseattribs(attribute *attribs)
{
  int val;
  int fullnamepos;

  if ((fullnamepos = attribs->getintattribute("fullname", &val)))
  {
    if (val) NameLength = 2;
  }

  int titlepos;
  if ((titlepos = attribs->getintattribute("title", &val, fullnamepos)))
    if (!val) NameLength=0;

  char *name;
  int disptitlepos;
  if ((disptitlepos = attribs->getstringattribute("disptitle", &name, fullnamepos)))
  {
    hasextname=1;
    extname = symbtable.toid(name);
  }
}

/****************************************************************************
				   BOXWIDGET
 ****************************************************************************/

boxwidget::boxwidget(attribute *attribs, displaybox *boxs) : symbolwidget(attribs, boxs)
{
/*
  TGroupBox* box;
  box=new TGroupBox((TComponent*)NULL);
  box->Ctl3D=true;
  scrollbox=new TScrollBox((TComponent*)NULL);
  scrollbox->BorderStyle=bsNone;
  //scrollbox->Align=alClient;
  scrollbox->Left=3;
  scrollbox->Top=13;
  box->InsertControl(scrollbox);
  DEFINECONTROL(box);
*/
  TGroupBox* box;
  TPanel* boxcontainer;
  boxcontainer = (new TBox((TComponent*)NULL))->ExtractControl(NULL);
  TBoxInfo* boxinfo=(TBoxInfo*)(boxcontainer->Tag);
  box = boxinfo->GroupBox;
  scrollbox = boxinfo->ScrollBox;
  DEFINECONTROL(boxcontainer);
  boxinfo->OnTypeChange=OnChangeType;
  boxinfo->OnResize=OnResize;
  boxinfo->OnUpdate=OnUpdate;
  boxinfo->OnExportToClipboard=OnExportToClipBoard;
  int tnameval;
  list<int> tname(&tnameval, (list<int> *) NULL);
  symbol *s;

  widgets=NULL;
  list<symbolwidget> **last=&widgets;
  for (list<symbolconstructor> *l = boxs->constructors; l != NULL; l = l->next)
  {
    tnameval = l->elem->getname();
    s = boxs->getlocalsymbol(&tname);
    if (s == NULL) {
      error(TYPE_ERROR, -1, NULL, "Can't find %s to create box %s.", symbtable.tostring(tnameval), boxs->getfullnametext());
      continue;
    }
    symbolwidget *wi=s->getwidget();
    TControl *co=wi->GetVclWidget(this);
    list<symbolwidget> *newlist=new list<symbolwidget>(wi,(list<symbolwidget>*)NULL);
    *last=newlist;
    last=&newlist->next;
    if (co!=NULL)
      scrollbox->InsertControl(co);
  }

  //Drag & Drop
  PREPARE_DRAG_DROP(boxcontainer);
  boxinfo->SpreadDragDrop();
}

void __fastcall boxwidget::OnExportToClipBoard()
{
  TMemoryStream* stream=new TMemoryStream();
  exportwidget(stream,0);
  char b=0;
  stream->Write((void*)&b,1);
  Clipboard()->SetTextBuf(((char*)stream->Memory));
  delete stream;
}

void boxwidget::getsizewanted(int& x,int& y,int& ox,int& oy)
{
  TBoxInfo* boxinfo = ((TBoxInfo*)(control->Tag));
  int margin;
  int xc,yc,oxc,oyc;
  control->Left=ox;
  control->Top=oy;

  xc=yc=oxc=oyc=0;
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next) {
    (l->elem->getsizewanted)(xc,yc,oxc,oyc);
    if (boxtype==BOX_H)
      oxc+=l->elem->control->Width;
    else oyc+=l->elem->control->Height;
  }

  if (((TBoxInfo*)control->Tag)->Auto_size) {
    if (boxinfo->Auto_size) margin=0; else margin=2;
    xc+=4+4+4+margin;
    control->Width=MAX(xc,40);
    control->Height=yc+=17+4+4+margin;
    y=MAX(y,oy+control->Height);
    x=MAX(x,ox+control->Width);
  } else
  {
    symbolwidget::getsizewanted(x,y,ox,oy);
  }
}

void boxwidget::setsize(int rx,int ry)
{
  TBoxInfo* boxinfo = ((TBoxInfo*)(control->Tag));
  int margin;
  if (boxinfo->Auto_size) margin=0; else margin=2;
  rx-=4+4+4+margin;
  ry-=17+4+4+margin;
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next) {
    if (boxtype==BOX_H) {
      l->elem->control->Height=ry;
      (l->elem->setsize)(l->elem->control->Width,ry);
      rx-=l->elem->control->Width;
    } else {
      l->elem->control->Width=rx;
      (l->elem->setsize)(rx,l->elem->control->Height);
      ry-=l->elem->control->Height;
    }
  }
}

void boxwidget::passtochildren(void (widget::*fn)())
{
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next)
    (l->elem->*fn)();
}

void boxwidget::passtochildren(void (widget::*fn)(void *), void *p)
{
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next)
    (l->elem->*fn)(p);
}

int boxwidget::RemoveChild(widget *sw)
{
  list<symbolwidget> **pw = &widgets;
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next) {
    if (l->elem==sw) {
      //Remove TControl from control list
      TControl *co=l->elem->GetVclWidget(this);
      if (co!=NULL)
        scrollbox->RemoveControl(co);
      //Remove widget from widget list
      //Memory Leak
      *pw = l->next;
      return 0;
    }
    pw = &(l->next);
  }
  return 0;
}

void boxwidget::InsertWidgetAfter(symbolwidget *current,symbolwidget *wi)
{
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next) {
    if (l->elem==current) {
      l->next=new list<symbolwidget>(wi,l->next);
      TControl *co=wi->GetVclWidget(this);
      if (co!=NULL)
        scrollbox->InsertControl(co);
      return;
    }
  }
}

void __fastcall boxwidget::DragDrop(System::TObject* Sender, System::TObject* Source, int X, int Y) {
  //Remove widget from precedent owner.
  symbolwidget *wi=(symbolwidget*)((WDragObject*)Source)->wi;
  widget *wiparent=wi->parent;
  boxwidget *parentbox=(boxwidget*)parent;
  int destroyed=1;
  if (wiparent)
    destroyed=wiparent->RemoveChild(wi);
  if (getparentwindow()==parent) {
      //Place it at the top of the box;
      TControl *co=wi->GetVclWidget(this);
      widgets=new list<symbolwidget>(wi,widgets);
      if (co!=NULL)
      scrollbox->InsertControl(co);
    }
  else {
    //after or inside the current object
    if ((parentbox)&&(
        ((Y>=control->Height-6)&&(parentbox->boxtype==BOX_V))
      ||((X>=control->Width-6)&&(parentbox->boxtype==BOX_H))
      )) {
      //Place it after the current box;
      parentbox->InsertWidgetAfter(this,wi);
    }
    else {
      //Place it at the top of the box;
      TControl *co=wi->GetVclWidget(this);
      widgets=new list<symbolwidget>(wi,widgets);
      if (co!=NULL)
        scrollbox->InsertControl(co);
    }
  }
  //Update window
  updatewindow();
  if (!destroyed)
    if (wiparent->getparentwindow()!=getparentwindow()) wiparent->updatewindow();
}

void boxwidget::draw()
{

  TBoxInfo* boxinfo = ((TBoxInfo*)(control->Tag));
  boxinfo->SetName(OnGetName(boxinfo->NameLength));
/*
  if (!(symbtable.isanonymous(s->getname()))) {
    ((TGroupBox*)control)->Caption=Format("%s",ARRAYOFCONST((symbtable.tostring(s->getname()))));
  }
  else {
    ((TGroupBox*)control)->Caption="";
  };
  */
}

void __fastcall boxwidget::DragOver(System::TObject* Sender, System::TObject* Source, int X, int Y, TDragState State, bool &Accept)
{
  symbolwidget::DragOver(Sender,Source,X,Y,State,Accept);
  if ((parent)&&(Accept)) {
    if (getparentwindow()==parent) {
      dragcursor=crDrag;
      return;
    }
    boxwidget* parentbox=(boxwidget*)parent;
    if (parentbox->boxtype==BOX_V) {
      if (Y>=control->Height-6) {
        //insert under current item
        dragcursor=crVSplit;
      }
    }
    else {
      if (X>=control->Width-6) {
        //insert after current item
        dragcursor=crHSplit;
      }
    }
  }
}

void boxwidget::exportwidget(TStream* stream,int indent)
{
  for (int i=0;i<indent;i++) stream->Write(" ",1);
  if (boxtype==BOX_H) stream->Write("hbox {\r\n",8);
    else stream->Write("vbox {\r\n",8);
  for (list <symbolwidget> *l = widgets; l != NULL; l = l->next)
    (l->elem->exportwidget(stream,indent+2));
  for (int i=0;i<indent;i++) stream->Write(" ",1);
  stream->Write("};\r\n",4);
}

void __fastcall boxwidget::OnChangeType()
{
  boxtype=(((TBoxInfo*)control->Tag)->Boxhv);
  updatewindow();
}



/****************************************************************************
                   hboxwidget
****************************************************************************/
hboxwidget::hboxwidget(attribute *attribs, displaybox *s) : boxwidget(attribs, s)
{
 boxtype=BOX_H;
 TBoxInfo* boxinfo = ((TBoxInfo*)(control->Tag));
 boxinfo->SetBoxhv(boxtype);
 parseattribs(attribs);
 boxinfo->SetNameLength(NameLength);
}

/****************************************************************************
                   vboxwidget
****************************************************************************/
vboxwidget::vboxwidget(attribute *attribs, displaybox *s) : boxwidget(attribs, s)
{
  boxtype=BOX_V;
  TBoxInfo* boxinfo = ((TBoxInfo*)(control->Tag));
  boxinfo->SetBoxhv(boxtype);
  parseattribs(attribs);
  boxinfo->SetNameLength(NameLength);
}

/****************************************************************************
                   plainsymbolwidget
 ****************************************************************************/

plainsymbolwidget::plainsymbolwidget(attribute *attribs, symbol *s) : symbolwidget(attribs, s)
{
  TLabel *label=new TLabel(NULL);
  label->Caption=Format("Symbol without its widget type :\n%s",
    ARRAYOFCONST((s->getfullnametext())));
  DEFINECONTROL(label);

  //Drag & Drop
  PREPARE_DRAG_DROP(label);
}

/****************************************************************************
                   pinwidget
****************************************************************************/

pinwidget::pinwidget(attribute *attribs, pin *p) : symbolwidget(attribs, p)
{
  TPanel* pinbox;
  pinbox=(new TPinBox((TComponent*)NULL))->ExtractControl(NULL);
  DEFINECONTROL(pinbox);
  //Drag & Drop
  PREPARE_DRAG_DROP(pinbox);
  TPinInfo* pininfo=(TPinInfo*)(pinbox->Tag);
  pininfo->SpreadDragDrop();
  pininfo->OnUpdate=OnDraw;
  parseattribs(attribs);
  pininfo->SetNameLength(NameLength);
}

void __fastcall pinwidget::OnDraw()
{
  draw();
}

void pinwidget::draw()
{
  pin *p = (pin *) getsymbol();
  TPinInfo* pininfo=(TPinInfo*)(control->Tag);
  pininfo->Label2->Caption=OnGetName(pininfo->NameLength);
  pininfo->VIN->Caption=Format("%8.6g",ARRAYOFCONST((p->GetVin())));
  pininfo->VOUT->Caption=Format("%8.6g",ARRAYOFCONST((p->GetVout())));
  pininfo->Z->Caption=Format("%8.6g",ARRAYOFCONST((p->GetZ())));
}

void pinwidget::getsizewanted(int& x,int& y,int& ox,int& oy)
{
  TPinInfo* pininfo=(TPinInfo*)(control->Tag);
  control->Height=17;
  control->Width=4+pininfo->Panel1->Width+pininfo->Label2->Width;
  symbolwidget::getsizewanted(x,y,ox,oy);
}

void pinwidget::setsize(int rx,int ry)
{
  control->Height=17;
}

/****************************************************************************
				   VARIABLEWIDGET
 ****************************************************************************/

variablewidget::variablewidget(attribute *attribs, variable *var) : symbolwidget(attribs, var)
{
  TPanel* variablebox;
  variablebox=(new TVariableViewer((TComponent*)NULL))->ExtractControl(NULL);
  control=variablebox;
  DEFINECONTROL(variablebox);
  //Drag & Drop
  PREPARE_DRAG_DROP(variablebox);
  variableinfo=(TVariableInfo*)(variablebox->Tag);
  variableinfo->SpreadDragDrop();
  variableinfo->OnUpdate=OnDraw;
  variableinfo->OnChange=OnChange;
  variableinfo->OnClear=OnClear;
  variableinfo->OnRandom=OnRandom;
  variableinfo->OnIncrement=OnIncrement;
  parseattribs(attribs);
  variableinfo->SetNameLength(NameLength);
  if (var->readonly) {
    variableinfo->BitBtn1->Font->Color=clInactiveCaption;
    variableinfo->BitBtn1->OnClick=NULL;
  }
  //control=new TPanel((TComponent*)NULL);
}

void variablewidget::draw()
{
  variable *var = (variable *) getsymbol();
  char *txt = (variableinfo->Sign) ? var->getsignedstringvalue(variableinfo->Base) : var->getstringvalue(variableinfo->Base);
  char *name = OnGetName(variableinfo->NameLength);
  variableinfo->SetName(name);
  variableinfo->SetValue(txt);
  control->Invalidate();
}

void __fastcall variablewidget::OnChange(char * s)
{
  ((variable *) getsymbol())->setvalue(s, variableinfo->Base);
  draw();
}

void __fastcall variablewidget::OnDraw()
{
  draw();
}

void __fastcall variablewidget::OnClear()
{
  ((variable *) getsymbol())->setvalue(0);
  draw();
}

void __fastcall variablewidget::OnRandom()
{
  int a;
  for (int i=0;i<2;i++) (((short int*)(&a))[i])=random(0x10000);
  ((variable *) getsymbol())->setvalue(a);
  draw();
}

void __fastcall variablewidget::OnIncrement(int i)
{
  variable *var = (variable *) getsymbol();
  var->setvalue(var->getvalue()+(long)i);
  draw();
}

void variablewidget::localparseattribs(attribute *attribs)
{
 int temp;
  if (attribs->getintattribute("base", &temp) && temp >= 2 && temp <= 36)
    variableinfo->Base = temp;
  if (attribs->getintattribute("signed", &temp))
    variableinfo->SetSign(temp);
  variable *var = (variable *) getsymbol();
  if (attribs->getintattribute("randomize", &temp)) OnRandom();
  if (attribs->getintattribute("clear", &temp)) var->setvalue(0);
  symbolwidget::localparseattribs(attribs);
}

void variablewidget::exportattribs(TStream* stream)
{
  char buffer[200];
  stream->Write("base = ",7);
  sprintf(buffer,"%d",variableinfo->Base);
  stream->Write(buffer,strlen(buffer));
  stream->Write(", signed = ",10);
  sprintf(buffer,"%d",variableinfo->Sign);
  stream->Write(buffer,strlen(buffer));
}

void variablewidget::getsizewanted(int& x,int& y,int& ox,int& oy)
{
  control->Height=17;
  control->Width=variableinfo->Panel1->Width+variableinfo->Label2->Width+variableinfo->SymbolWidth;
  symbolwidget::getsizewanted(x,y,ox,oy);
}

void variablewidget::setsize(int rx,int ry)
{
  control->Height=17;
}

/****************************************************************************
				   FLOATWIDGET
 ****************************************************************************/

realwidget::realwidget(attribute *attribs, realsymbol *r) : symbolwidget(attribs, r)
{
  TPanel* realbox;
  realbox=(new TRealViewer((TComponent*)NULL))->ExtractControl(NULL);
  DEFINECONTROL(realbox);
  //Drag & Drop
  PREPARE_DRAG_DROP(realbox);
  realinfo=(TRealInfo*)(realbox->Tag);
  realinfo->SpreadDragDrop();
  realinfo->OnUpdate=OnDraw;
  realinfo->OnChange=OnChange;
  parseattribs(attribs);
  if (r->readonly) {
    realinfo->BitBtn1->Font->Color=clInactiveCaption;
    realinfo->BitBtn1->OnClick=NULL;
  }
//  control=new TPanel((TComponent*)NULL);
}

void realwidget::draw()
{
  realsymbol *var = (realsymbol *) getsymbol();
  char txt[20];
  sprintf(txt,"%f",var->getvalue());
  char *name = OnGetName(realinfo->NameLength);
  realinfo->SetName(name);
  realinfo->SetValue(txt);
  control->Invalidate();
}

void __fastcall realwidget::OnChange(char * str)
{
  float f;
  sscanf(str,"%.10g",&f);
  ((realsymbol *) getsymbol())->setvalue(f);
  draw();
}

void __fastcall realwidget::OnDraw()
{
  draw();
}

void realwidget::getsizewanted(int& x,int& y,int& ox,int& oy)
{
  control->Height=17;
  control->Width=185;
  symbolwidget::getsizewanted(x,y,ox,oy);
}

void realwidget::setsize(int rx,int ry)
{
  control->Height=17;
}



/****************************************************************************
				   PLUGWIDGET
 ****************************************************************************/

plugwidget::plugwidget(attribute *attribs, plug *p) : vboxwidget(attribs, displaybox::getempty(BOX_V))
{
  widgets=NULL;
  for (list<symbol> *l = p->symbols; l != NULL; l = l->next)
  {
    symbolwidget *wi=l->elem->getwidget();
    widgets=new list<symbolwidget>(wi,widgets);
    TControl *co=wi->GetVclWidget(this);
    if (co)
      scrollbox->InsertControl(co);
  }
}

/****************************************************************************
				   DISASSEMBLYWIDGET
 ****************************************************************************/
disassemblywidget::disassemblywidget(attribute *attribs,disassembly *disasm) : symbolwidget (attribs,disasm)
{
  TPanel* disasmbox;
  disasmbox=(new TDisassemblyListViewer((TComponent*)NULL))->ExtractControl(NULL);
  TDisassemblyListInfo* disasminfo;
  disasminfo=(TDisassemblyListInfo*)(disasmbox->Tag);
  disasminfo->DisasmProc=DisasmProc;
  disasminfo->IsBreakPoint=BreakPointProc;
  disasminfo->GetIP=GetIPProc;
  disasminfo->ToggleBreakPoint=ToggleBreakPoint;
  disasminfo->OnResize=OnResize;
  disasminfo->OnChangeIP=OnChangeIP;
  disasminfo->OnRunToCursor=OnRunToCursor;
  disasminfo->OnClearBreakPoints=OnClearBreakPoints;
  int start,end;
  disasm->GetMemRange(&start,&end);
  disasminfo->SetMemRange(start,end);
  DEFINECONTROL(disasmbox);

  //Drag & Drop
  PREPARE_DRAG_DROP(disasmbox);
  disasminfo->SpreadDragDrop();
  parseattribs(attribs);
  disasminfo->SetNameLength(NameLength);
}

void disassemblywidget::localparseattribs(attribute *attribs)
{
  int w;
  symbolwidget::localparseattribs(attribs);
  if (attribs->getintattribute("width", &w)) control->Width=w;
  if (attribs->getintattribute("height", &w)) control->Height=w;
}

void __fastcall disassemblywidget::OnClearBreakPoints()
{
  disassembly *disasm=(disassembly *) getsymbol();
  disasm->ClearBreakpoints();
  draw();
}

void __fastcall disassemblywidget::OnRunToCursor(int addr)
{
  disassembly *disasm=(disassembly *) getsymbol();
  disasm->ToCursor(addr);
  Project.RunMode=RUN_GO;
  while (Simulate(1)>=0);
}

void __fastcall disassemblywidget::ToggleBreakPoint(int i)
{
  disassembly *disasm=(disassembly *) getsymbol();
  disasm->ToggleBreak(i);
}

void __fastcall disassemblywidget::DisasmProc(int Address, char * & progaddress, char * &opcode, char * &disasmcode, char * &sourcecode, int &opcodelen)
{
  disassembly *disasm=(disassembly *) getsymbol();
  disasmcode=disasm->GetLine(Address);
  sourcecode=disasm->GetSourceCode(Address);
  opcode=disasm->GetOpCode(Address);
  opcodelen=disasm->GetOpCodeLength(Address);
  progaddress=disasm->GetRealAddr(Address);
}

void __fastcall disassemblywidget::BreakPointProc(int Address, bool &isbreakpoint)
{
  isbreakpoint=((disassembly *) getsymbol())->IsBreak(Address);
}

void __fastcall disassemblywidget::GetIPProc(int &ip)
{
  ip=((disassembly *) getsymbol())->GetIP();
}

void  __fastcall disassemblywidget::OnChangeIP(char* str)
{
  unsigned long l;
  int i;
  i=setstringvalue(str,10,&l);
  if (i==0) ((disassembly *) getsymbol())->SetIP(l);
  draw();
}

void disassemblywidget::draw()
{
  TDisassemblyListInfo* disasminfo;
  disasminfo=(TDisassemblyListInfo*)(control->Tag);
  disasminfo->Update();
}

void disassemblywidget::exportattribs(TStream* stream)
{
  char buffer[200];
  sprintf(buffer,"width = %d, height = %d",control->Width,control->Height);
  stream->Write(buffer,strlen(buffer));
}


/****************************************************************************
				   OSCILLOWIDGET
 ****************************************************************************/
oscillowidget::oscillowidget(attribute *attribs,Oscillo *oscillo) : symbolwidget (attribs,oscillo)
{

  TPanel* oscillobox;
  oscillobox=(new TOscilloW((TComponent*)NULL))->ExtractControl(NULL);
//  oscillobox=new TDisassemblyList((TComponent*)NULL,oscillo->memsize);
  DEFINECONTROL(oscillobox);
  //Drag & Drop
  PREPARE_DRAG_DROP(oscillobox);
  TOscilloControls* oscillocontrols=(TOscilloControls*)(oscillobox->Tag);
  oscillocontrols->NbPoints=((Oscillo*)getsymbol())->memsize;
  oscillocontrols->GetPoints=OnGetPoints;
  oscillocontrols->OnResize=OnResize;
  oscillocontrols->GetCurveCount=OnGetCurveCount;
  oscillocontrols->GetCurvePos=OnGetCurvePos;
  parseattribs(attribs);
  oscillocontrols->ProcessCurves();
  oscillocontrols->SpreadDragDrop();
  lasttime=-1;
}

real* __fastcall oscillowidget::OnGetPoints(int i,char* &str)
{
  int j=0;
  for (list<pin_OscilloInput> *l = ((Oscillo*)getsymbol())->Inputs; l != NULL;l = l->next) {
    if (i==j) {
      str=l->elem->getfullnametext();
      return (l->elem->buffer);
    }
    j++;
  }
  return NULL;
}

int __fastcall oscillowidget::OnGetCurveCount()
{
  int j=0;
  for (list<pin_OscilloInput> *l = ((Oscillo*)getsymbol())->Inputs; l != NULL;l = l->next) {
    j++;
  }
  return j;
}

void oscillowidget::draw()
{
  TOscilloControls* oscillocontrols=(TOscilloControls*)(control->Tag);
  oscillocontrols->SetName(OnGetName(2));
  real time=Project.GetTime();
  if (time!=lasttime)
    oscillocontrols->PaintBox1->Invalidate();
  lasttime=time;
}

int __fastcall oscillowidget::OnGetCurvePos()
{
  return ((Oscillo*)getsymbol())->index;
}

void oscillowidget::localparseattribs(attribute *attribs)
{
  int w;
  if (attribs->getintattribute("width", &w)) control->Width=w;
  if (attribs->getintattribute("height", &w)) control->Height=w;
}

void oscillowidget::exportattribs(TStream* stream)
{
  char buffer[200];
  sprintf(buffer,"width = %d, height = %d",control->Width,control->Height);
  stream->Write(buffer,strlen(buffer));
}



/****************************************************************************
				   ARRAYWIDGET
 ****************************************************************************/
arraywidget::arraywidget(attribute *attribs, array *s): symbolwidget (attribs,s)
{
  TPanel* arraybox;
  arraybox=(new TArrayViewer((TComponent*)NULL))->ExtractControl(NULL);
  TArrayInfo* arrayinfo;
  arrayinfo=(TArrayInfo*)(arraybox->Tag);
  arrayinfo->OnGetData=OnGetData;
  arrayinfo->OnGetName=OnGetName;
  arrayinfo->OnResize=OnResize;
  arrayinfo->OnClear=OnClear;
  arrayinfo->OnRandom=OnRandom;
  arrayinfo->OnSetData=OnSetData;
  DEFINECONTROL(arraybox);
  //Drag & Drop
  PREPARE_DRAG_DROP(arraybox);
  arrayinfo->SpreadDragDrop();
  parseattribs(attribs);
  int start,end;
  s->getmemrange(&start,&end);
  arrayinfo->SetMemRange(start,end);
  arrayinfo->SetNameLength(NameLength);
}

void arraywidget::localparseattribs(attribute *attribs)
{
  int w;
  int updt;

  TArrayInfo* arrayinfo=(TArrayInfo*)(control->Tag);
  symbolwidget::localparseattribs(attribs);
  if (attribs->getintattribute("base", &w) && w >= 2 && w <= 36)
    arrayinfo->Base = w;
  if (attribs->getintattribute("signed", &w))
    arrayinfo->SetSign(w);
  if (attribs->getintattribute("reduced", &w)) {
    arrayinfo->Reduced1->Checked=(w);
    arrayinfo->Reduced=(w);

  }
  if (attribs->getintattribute("datawidth", &w))
  {
    updt=1;
    if (w == 8) {
      arrayinfo->DataSize=1;
      arrayinfo->Byte1->Checked=true;
    }
    if (w == 16) {
      arrayinfo->DataSize=2;
      arrayinfo->Word1->Checked=true;
    }
    if (w == 32) {
      arrayinfo->DataSize=4;
      arrayinfo->DWord1->Checked=true;
    }
  }
  if (attribs->getintattribute("columns", &w)) {
    updt=1;
    arrayinfo->LineSize=w;
    switch (arrayinfo->LineSize) {
      case 1:arrayinfo->N1line1->Checked=true;break;
      case 2:arrayinfo->N2line1->Checked=true;break;
      case 4:arrayinfo->N4line1->Checked=true;break;
      case 8:arrayinfo->N8line1->Checked=true;break;
      case 16:arrayinfo->N16line1->Checked=true;break;
    }
  }
  if (attribs->getintattribute("width", &w)) control->Width=w;
  if (attribs->getintattribute("height", &w)) control->Height=w;
  int start,end;
  ((array*)getsymbol())->getmemrange(&start,&end);
  if (updt) arrayinfo->SetMemRange(start,end);
}

char* __fastcall arraywidget::OnGetData(int addr,int size) {
  //
  array* myarray=(array*)getsymbol();
  TArrayInfo* arrayinfo=(TArrayInfo*)(control->Tag);

  if (myarray->cangetvalue(addr,size)) {
    if (!arrayinfo->Sign) {
      switch (size) {
        case 1:return getstringvalue(myarray->getcharvalue(addr),size*8,arrayinfo->Base);
        case 2:return getstringvalue(myarray->getwordvalue(addr),size*8,arrayinfo->Base);
        case 4:return getstringvalue(myarray->getdwordvalue(addr),size*8,arrayinfo->Base);
      }
    } else {
      switch (size) {
        case 1:return getsignedstringvalue(myarray->getcharvalue(addr),size*8,arrayinfo->Base);
        case 2:return getsignedstringvalue(myarray->getwordvalue(addr),size*8,arrayinfo->Base);
        case 4:return getsignedstringvalue(myarray->getdwordvalue(addr),size*8,arrayinfo->Base);
      }
    }
  }
  return "";
}

void __fastcall arraywidget::OnSetData(int addr,char* value, int size)
{
  array* myarray=((array*)getsymbol());
  TArrayInfo* arrayinfo=(TArrayInfo*)(control->Tag);
  unsigned long l;
  int i;
  i=setstringvalue(value,arrayinfo->Base,&l);
  if (i==0) {
    switch (size) {
      case 1:myarray->setcharvalue(addr,l);
      case 2:myarray->setwordvalue(addr,l);
      case 4:myarray->setdwordvalue(addr,l);
    }
  }
}

void arraywidget::draw()
{
  TArrayInfo* arrayinfo=(TArrayInfo*)(control->Tag);
  arrayinfo->StringGrid1->Invalidate();
}

void __fastcall arraywidget::OnClear()
{
  array* myarray=((array*)getsymbol());
  int start,end;
  myarray->getmemrange(&start,&end);
  for (int i=start;i<end;i++) myarray->setcharvalue(i,0);
  draw();
}

void __fastcall arraywidget::OnRandom()
{
  array* myarray=((array*)getsymbol());
  int start,end;
  myarray->getmemrange(&start,&end);
  for (int i=start;i<end;i++) myarray->setcharvalue(i,random(256));
  draw();
}

void arraywidget::exportattribs(TStream* stream)
{
  char buffer[200];
  TArrayInfo* arrayinfo=(TArrayInfo*)(control->Tag);
  sprintf(buffer,"base = %d, signed = %d",arrayinfo->Base,arrayinfo->Sign);
  stream->Write(buffer,strlen(buffer));
  sprintf(buffer,", columns = %d, datawidth = %d",arrayinfo->LineSize,arrayinfo->DataSize*8);
  stream->Write(buffer,strlen(buffer));
  sprintf(buffer,", width = %d, height = %d",control->Width,control->Height);
  stream->Write(buffer,strlen(buffer));
  sprintf(buffer,", reduced = %d",arrayinfo->Reduced);
  stream->Write(buffer,strlen(buffer));
}


/****************************************************************************
				   PROJECTWIDGET
 ****************************************************************************/

void recurseadd(TTreeView* tree,TTreeNode* parent,symbol *s,int i)
{
  TTreeNode* newparent;
  if (i>=0) {
    newparent=tree->Items->AddChild(parent,symbtable.tostring(s->getname()));
    newparent->Data=s;
  } else {
    newparent=parent;
    i=1;
  }
  if ((i>0)&&(!newparent->HasChildren)) {
    for (list<symbol> *l = s->getapparentsymbols(); l != NULL; l = l->next)
      if (/*displayanonymous ||*/ !symbtable.isanonymous(l->elem->getname()))
        recurseadd(tree,newparent, l->elem,i-1);
  }
}

projectwidget::projectwidget(attribute *attribs, symbol *s):symbolwidget(attribs,s)
{
  TPanel* projectbox;
  projectbox=(new TProjectViewer((TComponent*)NULL))->ExtractControl(NULL);
  TProjectInfo* projectinfo=(TProjectInfo*)(projectbox->Tag);
  projectinfo->OnTreeExpand=TreeExpand;
  projectinfo->OnResize=OnResize;

  DEFINECONTROL(projectbox);

  //Drag & Drop
  PREPARE_DRAG_DROP(projectbox);
  projectinfo->SpreadDragDrop();
  projectinfo->TreeView1->OnStartDrag=StartDrag2;
  parseattribs(attribs);
}

void projectwidget::draw()
{
  TProjectInfo* projectinfo=(TProjectInfo*)(control->Tag);
  projectinfo->GroupBox1->Caption=getsymbol()->getfullnametext();
  if (projectinfo->TreeView1->Items->Count==0) {
    recurseadd(projectinfo->TreeView1,NULL,getsymbol(),1);
    projectinfo->TreeView1->SortType=stBoth;
  }
}

void __fastcall projectwidget::TreeExpand(TTreeNode *Node)
{
  TProjectInfo* projectinfo=(TProjectInfo*)(control->Tag);

  TTreeNode* CurrentNode=Node->getFirstChild();
  while (CurrentNode) {
    recurseadd(projectinfo->TreeView1,CurrentNode,(symbol*)(CurrentNode->Data),-1);
    CurrentNode=CurrentNode->GetNext();
  }
  projectinfo->TreeView1->SortType=stBoth;
}

void __fastcall projectwidget::StartDrag2(System::TObject* Sender, TDragObject* &DragObject)
{
  TProjectInfo* projectinfo=(TProjectInfo*)(control->Tag);

  TTreeNode* Node=projectinfo->TreeView1->Selected;
  if (Node) {
    DragObject=new WDragObject();
    char* txt=Node->Text.c_str();
    symbol* symb=(symbol*)Node->Data;
    lastwidget=((WDragObject*)DragObject)->wi=symb->getwidget();
    ((WDragObject*)DragObject)->todestroy=1;
  } else {
    control->EndDrag(false);
    lastwidget=NULL;
  }
}

//---------------------------------------------------------------------------
#pragma package(smart_init)
