/*
  DUST: Simulator for circuits containing a SX microcontroller.
  Copyright (C) 1999 Blaise Gassend <blaise at gassend dot com> 
  and Franois-Marie Lefevere <francois-marie dot lefevere at m4x dot org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#ifndef widget_winH
#define widget_winH
#include "VariableUnit.hpp"
#include "RealUnit.hpp"
#include "testunit.h"

class window;
class symbolwidget;
class symbolborderwidget;
class boxwidget;
class widget;

void QuitProgram();

class WDragObject: public TDragObject {
public:
  widget* wi;
  int todestroy;
  TCursor __fastcall GetDragCursor(bool Accepted, int X, int Y);
  void __fastcall Finished(System::TObject* Target, int X, int Y, bool Accepted);
};

//******************************************************************************
class widget
{
protected:
  virtual void draw() {};
  virtual void passtochildren(void (widget::*fn)()) {};
  virtual void passtochildren(void (widget::*fn)(void *), void *p) {};
public:
  attribute  *initattribs;
  TControl *control;
  widget *parent;
  widget(attribute *attribs);
  virtual void localparseattribs(attribute *attribs) {};
  virtual ~widget();
  virtual void refresh();
  virtual int RemoveChild(widget *sw) { return 0; };
  virtual void InsertWidgetAfter(symbolwidget *current,symbolwidget *wi) {};
  virtual void updatewindow() {};
  virtual widget* getparentwindow() { return NULL; }
  void parseattribs(attribute *attribs);
  bool isparent(widget* wi) { if (parent) return (parent==wi)||(parent->isparent(wi)); else return false; }

  virtual void exportwidget(TStream* stream,int indent)=0;
  virtual void exportattribs(TStream* stream) {};
};

//******************************************************************************
class window : public widget
{
protected:
  bool destroying;
  symbolwidget *sw;
  void __fastcall ContainerResize(TObject *Sender);
  void __fastcall WindowDestroy(TObject *Sender);
  void passtochildren(void (widget::*fn)());
  void passtochildren(void (widget::*fn)(void *), void *p);
public:
  window(attribute *attribs, symbolwidget *);
  ~window();
  void updatewindow();
  widget* getparentwindow() { return this;}
  int RemoveChild(widget *w);
  void exportwidget(TStream* stream,int indent);
  void localparseattribs(attribute *attribs);
};

//******************************************************************************
class symbolwidget : public widget
{
protected:
  int hasextname;
  int NameLength;
  int extname;
  virtual void __fastcall StartDrag(System::TObject* Sender, TDragObject* &DragObject);
  virtual void __fastcall EndDrag(System::TObject* Sender, System::TObject* Target, int X, int Y);
  virtual void __fastcall DragOver(System::TObject* Sender, System::TObject* Source, int X, int Y, TDragState State, bool &Accept);
  virtual void __fastcall DragDrop(System::TObject* Sender, System::TObject* Source, int X, int Y);
  virtual void __fastcall MouseDown(System::TObject* Sender, TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
  virtual void __fastcall DblClick(System::TObject* Sender) {};
  char* __fastcall OnGetName(int NameLength);
  void __fastcall OnResize() { updatewindow();};
public:
  symbol *sy;
  symbolwidget(attribute *attribs, symbol *s);
  symbol *getsymbol(void) { return sy; }
  virtual void getsizewanted(int& x,int& y,int& ox,int& oy);
  virtual void setsize(int rx,int ry);
  TControl *GetVclWidget(widget *p) { parent = p; return control; }
  virtual void Resize(void) {};
  void updatewindow() { if (parent) parent->updatewindow(); }
  widget* getparentwindow() { if (parent) return parent->getparentwindow(); else return NULL; }
  void localparseattribs(attribute *attribs);
  void exportwidget(TStream* stream,int indent);
};

//******************************************************************************
class symbolborderwidget : public symbolwidget
{
};

//******************************************************************************
class plainsymbolwidget : public symbolwidget
{
public:
  plainsymbolwidget(attribute *attribs, symbol *s);
};

//******************************************************************************
class boxwidget : public symbolwidget
{
protected:
  TScrollBox *scrollbox;
  void draw();
  void passtochildren(void (widget::*fn)());
  void passtochildren(void (widget::*fn)(void *), void *p);
  void __fastcall DragDrop(System::TObject* Sender, System::TObject* Source, int X, int Y);
  void __fastcall DragOver(System::TObject* Sender, System::TObject* Source, int X, int Y, TDragState State, bool &Accept);
  void __fastcall OnChangeType();
  void __fastcall OnUpdate() { draw();};
  void __fastcall OnExportToClipBoard();
public:
  int boxtype;
  list<symbolwidget> *widgets;
  boxwidget(attribute *attribs, displaybox *boxs);
  void getsizewanted(int& x,int& y,int& ox,int& oy);
  void setsize(int rx,int ry);
  int RemoveChild(widget *sw);
  void InsertWidgetAfter(symbolwidget *current,symbolwidget *wi);
  void exportwidget(TStream* stream,int indent);
};
//******************************************************************************
class hboxwidget : public boxwidget
{
public:
  hboxwidget(attribute *attribs, displaybox *s);
};
//******************************************************************************
class vboxwidget : public boxwidget
{
public:
  vboxwidget(attribute *attribs, displaybox *s);
};
//******************************************************************************
class pinwidget : public symbolwidget
{
protected:
  void draw();
  void __fastcall OnDraw();
public:
  pinwidget(attribute *attribs, pin *p);
  void getsizewanted(int& x,int& y,int& ox,int& oy);
  void setsize(int rx,int ry);
};

//******************************************************************************
class variablewidget : public symbolwidget
{
protected:
  TVariableInfo* variableinfo;
  void draw();
  void __fastcall OnDraw();
  void __fastcall OnChange(char * s);
  void __fastcall OnClear();
  void __fastcall OnRandom();
  void __fastcall OnIncrement(int i);
  void localparseattribs(attribute *attribs);
public:
  variablewidget(attribute *attribs, variable *var);
  void exportattribs(TStream* stream);
  void getsizewanted(int& x,int& y,int& ox,int& oy);
  void setsize(int rx,int ry);
};

//******************************************************************************
class realwidget : public symbolwidget
{
protected:
  TRealInfo * realinfo;
  void draw();
  void __fastcall OnDraw();
  void __fastcall OnChange(char * str);
public:
  realwidget(attribute *attribs, realsymbol *r);
  void getsizewanted(int& x,int& y,int& ox,int& oy);
  void setsize(int rx,int ry);
};

//******************************************************************************
class plugwidget : public vboxwidget
{
public:
  plugwidget(attribute *attribs, plug *p);
};
//******************************************************************************
class disassemblywidget : public symbolwidget
{
protected:
  void draw();
  void __fastcall DisasmProc(int Address, char * & progaddress, char * &opcode, char * &disasmcode,char * &sourcecode, int &opcodelen);
  void __fastcall BreakPointProc(int Address, bool &isbreakpoint);
  void __fastcall GetIPProc(int &ip);
  void __fastcall ToggleBreakPoint(int i);
  void __fastcall OnChangeIP(char* str);
  void __fastcall OnClearBreakPoints();
  void __fastcall OnRunToCursor(int addr);
  void localparseattribs(attribute *attribs);
public:
  disassemblywidget(attribute *attribs,disassembly *disasm);
  void exportattribs(TStream* stream);
};
//******************************************************************************
class oscillowidget : public symbolwidget
{
protected:
  real lasttime;
  void draw();
  real* __fastcall OnGetPoints(int i,char* &str);
  int __fastcall OnGetCurveCount();
  int __fastcall OnGetCurvePos();
  void localparseattribs(attribute *attribs);
public:
  oscillowidget(attribute *attribs,Oscillo *oscillo);
  void exportattribs(TStream* stream);
};

//******************************************************************************
class arraywidget : public symbolwidget
{
protected:
  void draw();
  char* __fastcall OnGetData(int addr,int size);
  void __fastcall OnClear();
  void __fastcall OnRandom();
  void __fastcall OnSetData(int addr,char* value, int size);
  void localparseattribs(attribute *attribs);
public:
  arraywidget(attribute *attribs, array *s);
  void exportattribs(TStream* stream);
};

//******************************************************************************
class projectwidget : public symbolwidget
{
protected:
  widget *lastwidget;
  void draw();
  void __fastcall TreeExpand(TTreeNode *Node);
  TDragObject* __fastcall GetDragObject(TTreeNode *Node);
  void __fastcall StartDrag2(System::TObject* Sender, TDragObject* &DragObject);
public:
  projectwidget(attribute *attribs, symbol *s);
};

//---------------------------------------------------------------------------
#endif

